/*******************************************************************************
* Title: IR_Sensors.c                                                                      
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                                        
*                                                                              
* Description:                                                                 
* This file containts the functions that are used to calibrate and read the IR                                                                             
* sensors on the beer pong table. This allows us to detect when a cup has been                                                                             
* removed from or placed on a pod, and it allows us to detect when a ball has
* been dropped into either of the ballwashers.     
*  
* In accordance with the schematic, each side of the BPT has its own dedicated ADC
* port for its 74HC4051 chips. Chips #1 and #2 are on AN0, whereas #3 and #4 are 
* on AN1.
*
* IR_TX1 controls the transmitters that coincide with the IR sensors on chips #1 and
* #2. IR_TX2 controls the transmitters that coincide with the IR sensors on chips #3
* and #4.   
*                                                                                          
* The IR sensor values will be stored in a 24-byte variable array as follows:
*
* Bytes:
*
*     1st 10x RGB Pods (Master Side)
* ----------------------------------
*  0 -> RGB Pod #1  - IR Sensor #1  
*  1 -> RGB Pod #2  - IR Sensor #2  
*  2 -> RGB Pod #3  - IR Sensor #3  
*  3 -> RGB Pod #4  - IR Sensor #4  
*  4 -> RGB Pod #5  - IR Sensor #5  
*  5 -> RGB Pod #6  - IR Sensor #6  
*  6 -> RGB Pod #7  - IR Sensor #7  
*  7 -> RGB Pod #8  - IR Sensor #8  
*  8 -> RGB Pod #9  - IR Sensor #9  
*  9 -> RGB Pod #10 - IR Sensor #10    
*
*      2nd 10x RGB Pods (Secondary Side)
* --------------------------------------
*  10 -> RGB Pod #11  - IR Sensor #11  
*  11 -> RGB Pod #12  - IR Sensor #12  
*  12 -> RGB Pod #13  - IR Sensor #13  
*  13 -> RGB Pod #14  - IR Sensor #14  
*  14 -> RGB Pod #15  - IR Sensor #15  
*  15 -> RGB Pod #16  - IR Sensor #16  
*  16 -> RGB Pod #17  - IR Sensor #17  
*  17 -> RGB Pod #18  - IR Sensor #18  
*  18 -> RGB Pod #19  - IR Sensor #19  
*  19 -> RGB Pod #20  - IR Sensor #20 
*
*      Ball Washer IR Sensors (Master Side)
* -----------------------------------------
*  20 -> IR Ballwasher #1 Entry 
*  21 -> IR Ballwasher #1 Exit   
*
*      Ball Washer IR Sensors (Secondary Side)
* --------------------------------------------
*  22 -> IR Ballwasher #2 Entry 
*  23 -> IR Ballwasher #2 Exit   
*                                                                                                                         
*******************************************************************************/

#ifndef IR_SENSORS_C
#define IR_SENSORS_C

#include "Main_Includes.h"
#include "IR_Sensors.h"
#include "ADC_Setup.h"
#include "Delay_Setup.h"

/*************************************************
*               Global Variables                 *
*************************************************/
extern volatile UINT16 IR_cal[24];
extern volatile UINT16 IR_value[24];
extern volatile UINT32 sensor_bits;

/*******************************************************************************
* Function: Sensor_Calibration(void)                                                                    
*                                                                              
* Variables:                                                                   
* IR_cal[24] -> All calibration analog values stored in this global UINT16 array                                                                          
*                                                                              
* Description:                                                                 
* This function will calibrate the IR sensors by reading each sensor 'n' amount of                                                                            
* times and calculating the average reading. The amount of times that each sensor                                                                             
* be read is determined by CAL_DIV located in the header file. Since average is a
* 16-bit variable, CAL_DIV should not go above the value of 64.
* 
* Max ADC Value = 1023
*                  
* average = 1023 * CAL_DIV_MAX (64)
*         = 65472
*                  
* If CAL_DIV is any higher than 64 and your ADC values are at max value for all of 
* the readings, you will overflow the 16-bit 'average' variable. If you need more
* than 64 readings to average, you can change 'average' to a 32-bit varaiable.                                                                            
*******************************************************************************/
void Sensor_Calibration(void)
{
	UINT8 i;
	UINT8 j;
	UINT8 spot;
	UINT16 average = 0;
  
  //Enable all IR transmitters; Tried enabling one at a time but the ADC values
  //were not as consistent, leave them on all the time. Allow a few ms to stabilize. 
  IR_TX1 = 1;
  IR_TX2 = 1;
  Delay_ms(30);
  
  //Enable the 1st 74HC4051; Get ready to read IR_Sensors 0 - 7
  IR_C1_EN = 0;
  
  //Set the ADC channel in accordance with the sensors to be measured
  ADC_CHANNEL(0);
  Delay_us(50);
  
  //Read the first 8 sensors which are connected to 74HC4051 #1 (Master Side) 
  for (i = 0;i < 8;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= (i << 13);
    
    //Read the sensor value 'CAL_DIV' amount of times, saving the value each time
    for (j = 0;j < CAL_DIV;j++)
    {
      Delay_us(60);
      average += ADC_Read();
    }  
    
    //This calculates the correct location in the array for the sensor IR_cal
    //This ensures that the RGB Pod coincides with its infrared sensor in the packed IR_cal
    spot = 19 - i;
    
    //Calculate the average reading then reset the 'average' variable
    IR_cal[spot] = average / CAL_DIV;
    average = 0;
  }
  
  //Deselect chip #1 and select chip #2
  IR_C1_EN = 1;
  IR_C2_EN = 0; 
  
  //On chip #2, the first 2x inputs (X0 & X1) correspond to sensors #9 and #10
  for (i = 0;i < 2;i++)
  {
    LATB &= 0x1FFF;
    LATB |= (i << 13);
    
    //Read the sensor value 'CAL_DIV' amount of times, saving the value each time
    for (j = 0;j < CAL_DIV;j++)
    {
      Delay_us(60);
      average += ADC_Read();
    }    
    
    //This calculates the correct location in the array for the sensor IR_cal
    //This ensures that the RGB Pod coincides with its infrared sensor in the packed IR_cal
    spot = 19 - (i+8);
    
    //Calculate the average reading then reset the 'average' variable
    //Pack the new IR_cal into the array following the format stated above in the intro
    IR_cal[spot] = average / CAL_DIV;
    average = 0;
  }
  
  //On chip #2, the next 2x inputs (X3 & X4) correspond to BW sensors entry and exit #1
  for (i = 0;i < 2;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= ((i+2) << 13);
    
    //Read the sensor value 'CAL_DIV' amount of times, saving the value each time
    for (j = 0;j < CAL_DIV;j++)
    {
      Delay_us(60);
      average += ADC_Read();
    }  
    
    //Calculate the average reading then reset the 'average' variable
    //Pack the new IR_cal into the array following the format stated above in the intro
    IR_cal[i+22] = average / CAL_DIV;
    average = 0;
  }
  
  //Deselect chip #2, Select chip #3 (Same as above, except for secondary side now)
  IR_C2_EN = 1;  
  IR_C3_EN = 0;
  
  //Set the ADC channel in accordance with the sensors to be measured
  ADC_CHANNEL(1);
  Delay_us(50);
    
  //Read the 8 sensors which are connected to 74HC4051 #3 (Secondary Side)  
  for (i = 0;i < 8;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= (i << 13);
    
    //Read the sensor value 'CAL_DIV' amount of times, saving the value each time
    for (j = 0;j < CAL_DIV;j++)
    {
      Delay_us(60);
      average += ADC_Read();
    } 
    
    //This calculates the correct location in the array for the sensor IR_cal
    //This ensures that the RGB Pod coincides with its infrared sensor in the packed IR_cal
    spot = 19-(i+10);
     
    //Calculate the average reading then reset the 'average' variable
    //Pack the new IR_cal into the array following the format stated above in the intro
    IR_cal[spot] = average / CAL_DIV;
    average = 0;
  }
  
  //Deselect chip #3 and select chip #4
  IR_C3_EN = 1;
  IR_C4_EN = 0; 
  
  //On chip #4, the first 2x inputs (X0 & X1) correspond to sensors #19 and #20
  for (i = 0;i < 2;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= (i << 13);
    
    //Read the sensor value 'CAL_DIV' amount of times, saving the value each time
    for (j = 0;j < CAL_DIV;j++)
    {
      Delay_us(60);
      average += ADC_Read();
    } 
    
    //This calculates the correct location in the array for the sensor IR_cal
    //This ensures that the RGB Pod coincides with its infrared sensor in the packed IR_cal
    spot = 19-(i+18); 
    
    //Calculate the average reading then reset the 'average' variable
    //Pack the new IR_cal into the array following the format stated above in the intro    
    IR_cal[spot] = average / CAL_DIV;
    average = 0;
  }
  
  //On chip #4, the next 2x inputs (X3 & X4) correspond to BW sensors entry and exit #2
  for (i = 0;i < 2;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= ((i+2) << 13);
    
    //Read the sensor value 'CAL_DIV' amount of times, saving the value each time
    for (j = 0;j < CAL_DIV;j++)
    {
      Delay_us(60);
      average += ADC_Read();
    }  
    
    //Calculate the average reading then reset the 'average' variable
    //Pack the new IR_cal into the array following the format stated above in the intro    
    IR_cal[i+20] = average / CAL_DIV;
    average = 0;
  }
  
  //Deselect Chip #4
  IR_C4_EN = 1;  
}	

/*******************************************************************************
* Function: Update_Ball_Washers(UINT16 *data)                                                                     
*                                                                              
* Variables:                                                                   
* *data -> Stores the 4x Ball Washer IR Sensor values                                                                          
*                                                                              
* Description:                                                                 
* This function will update the 4x ballwasher IR sensors and pack the data as
* follows:                                                                           
*                                                                              
* Bytes:                                                                           
*                                                                              
* 0 - BW Entry Sensor #1                                                       
* 1 - BW Exit Sensor  #1                                                       
* 2 - BW Entry Sensor #2                                                       
* 3 - BW Exit Sensor  #2     
*                                                   
*******************************************************************************/
void Update_Ball_Washers(UINT16 *data)
{ 
  UINT8 i;
  
  //Ensure that transmitters are on (May still be on from previous function call)  
  IR_TX1 = 1;
  IR_TX2 = 1;
  
  //Select chip #2 in order to read the first 2x sensor values (Connected to AN0)
  IR_C2_EN = 0; 
  ADC_CHANNEL(0);
  Delay_us(50);

  //Read Entry and Exit sensors #1
  for (i = 0;i < 2;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= ((i+2) << 13);
    Delay_us(IR_DELAY);
    
    //Read sensor value
    data[i] = ADC_Read();
  }
  
  //Deselect chip #2; Select chip #4 in order to read the next 2x BW sensor values
  IR_C2_EN = 1;   
  IR_C4_EN = 0; 
  
  //Change to AN1 as that is what chip #4 is connected
  ADC_CHANNEL(1);
  Delay_us(50);
  
  //Read Entry and Exit sensors #2
  for (i = 0;i < 2;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= ((i+2) << 13);
    Delay_us(IR_DELAY);
    
    //Read sensor value
    data[i+2] = ADC_Read();
  }
  
  //Deselect chip #4
  IR_C4_EN = 1;  
}  

/*******************************************************************************
* Function: Update_Sensors(void)                                                                  
*                                                                              
* Variables:                                                                   
* IR_value[24] -> The analog values will be stored in this global UINT16 array                                                                         
*                                                                              
* Description:                                                                 
* This function will update the value of the IR sensors which can then be compared
* to the calibrated values to see if a detection has been made. (Cup moved, ball
* put in washer)                                                                         
*******************************************************************************/
void Update_Sensors(void)
{ 
  UINT8 i;
  
  //Ensure that transmitters are on (May still be on from previous function call) 
  IR_TX1 = 1;
  IR_TX2 = 1;
  
  //Enable the 1st 74HC4051; Get ready to read IR_Sensors 0 - 7
  IR_C1_EN = 0;
  
  //Set the ADC channel in accordance with the sensors to be measured
  ADC_CHANNEL(0);
  Delay_us(IR_DELAY);
  
  //Read the first 8 sensors which are connected to 74HC4051 #1 (Master Side)  
  for (i = 0;i < 8;i++)
  {    
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= (i << 13);
    Delay_us(IR_DELAY);
    
    //Read the sensor value
    //Pack the new IR_value into the array following the format stated above in the intro 
    IR_value[19-i] = ADC_Read();
  }
  
  //Deselect chip #1 and select chip #2
  IR_C1_EN = 1;
  IR_C2_EN = 0; 
  
  //On chip #2, the first 2x inputs (X0 & X1) correspond to sensors #9 and #10
  for (i = 0;i < 2;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= (i << 13);
    Delay_us(IR_DELAY);
    
    //Read the sensor value
    //Pack the new IR_value into the array following the format stated above in the intro 
    IR_value[19-(i+8)] = ADC_Read();
  }

  //On chip #2, the next 2x inputs (X3 & X4) correspond to BW sensors entry and exit #1
  for (i = 0;i < 2;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= ((i+2) << 13);
    Delay_us(IR_DELAY);
    
    //Read the sensor value
    //Pack the new IR_value into the array following the format stated above in the intro 
    IR_value[i+22] = ADC_Read();
  }
   
  //Deselect chip #2, Select chip #3 (Same as above, except for secondary side now)
  IR_C2_EN = 1;  
  IR_C3_EN = 0;
 
  //Set the ADC channel in accordance with the sensors to be measured 
  ADC_CHANNEL(1);
  Delay_us(IR_DELAY);
   
  //Read the 8 sensors which are connected to 74HC4051 #3 (Secondary Side)  
  for (i = 0;i < 8;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= (i << 13);
    Delay_us(IR_DELAY);
    
    //Read the sensor value
    //Pack the new IR_value into the array following the format stated above in the intro 
    IR_value[19-(i+10)] = ADC_Read();
  }
    
  //Deselect chip #3 and select chip #4
  IR_C3_EN = 1;
  IR_C4_EN = 0; 

  //On chip #4, the first 2x inputs (X0 & X1) correspond to sensors #19 and #20 
  for (i = 0;i < 2;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= (i << 13);
    Delay_us(IR_DELAY);
    
    //Read the sensor value
    //Pack the new IR_value into the array following the format stated above in the intro 
    IR_value[19-(i+18)] = ADC_Read();
  }
  
  //On chip #4, the next 2x inputs (X3 & X4) correspond to BW sensors entry and exit #2
  for (i = 0;i < 2;i++)
  {
    //Set pin select pins to corresponding input
    LATB &= 0x1FFF;
    LATB |= ((i+2) << 13);
    Delay_us(IR_DELAY);
    
    //Read the sensor value
    //Pack the new IR_value into the array following the format stated above in the intro  
    IR_value[i+20] = ADC_Read();
  }
  
  //Deselect chip #4
  IR_C4_EN = 1;  
}  

/*******************************************************************************
* Function: Pack_Sensor_Data(void)                                                              
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:  
* This function will compare the IR sensor calibrated values with their current
* values to determine if anything has changed. The function will then return a
* 32-bit variable where each sensor is represented as a bit. There are 20 RGB pod
* sensors and 4 ball washer sensors, therefore we will only use a total of 24 bits.
* Pods #1 - #20 are represented by bits #0 - #19 respectively. Ball washer #1 entry
* sensor is represented by bit #20, BW #1 exit sensor is represented by bit #21,
* BW #2 entry sensor is represented by bit #22 and BW #2 exit sensor is represented
* by bit #23. If the bit is set (1) it has detected an object, if the bit is 
* cleared (0) then there is no object detected.     
*******************************************************************************/
UINT32 Pack_Sensor_Data(void)
{
	UINT8 i;
  UINT32 sensor_detect = 0;
	
	//Loop through all 20 pod IR sensors and determine if there is an object detected
  for (i = 0;i <20;i++)
  {   
    //If an object is detected, set the bit that represents that sensor
    if (IR_value[i] > (IR_cal[i] + IR_THRESHOLD)) 
	      sensor_detect |= ((UINT32)1 << i);      
  } 
	
	//Loop through all 4 Ball Washer IR sensors and determine if there is an object detected
  for (i = 20;i < 24;i++)
  {   
    //If an object is detected, set the bit that represents that sensor
    if (IR_value[i] > (IR_cal[i] + BW_THRESHOLD)) 
	      sensor_detect |= ((UINT32)1 << i);      
  } 
  
  //Return the data 
  return sensor_detect;
}
    
#endif 
