/*******************************************************************************
* Title: Miscellaneous.c                                                       
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                                       
*                                                                              
* Description:                                                                 
* This file contains various functions that pertain to the operation of the LED grid,                                                                             
* the RGB pods, the LED rings and the ball washers.                                     
*******************************************************************************/
   
#ifndef MISCELLANEOUS_C
#define MISCELLANEOUS_C  

#include "Main_Includes.h"
#include "Delay_Setup.h"
#include "Miscellaneous.h"
#include <stdlib.h>
#include "TLC5940_Setup.h"
#include "LED_Graphics.h"
#include "HT1632C_Setup.h"
#include "Pong_Game.h"

/*************************************************
*               Global Variables                 *
*************************************************/
extern volatile T16_FLAG FLAG1;

extern volatile UINT8 VU_On;
extern volatile UINT8 start[5];
extern volatile UINT8 error_code;
extern volatile UINT8 RGB_data[120];
extern volatile UINT8 seq[SEQ_AMOUNT];

extern volatile UINT16 LEDring[16];
extern volatile UINT16 ring_update;
extern volatile UINT16 LED_data[32];

extern volatile UINT32 count32;
extern volatile UINT32 pod_update;
extern volatile UINT32 sensor_bits;

extern volatile RGB PODn[20];
extern volatile const RGB COLOR[12];

extern volatile RGB_FADE RGB_diff[20];
extern volatile LED_RING ring_diff[16]; 	
	
/*******************************************************************************
* Function: Animate_On_Detection(void)                                                             
*                                                                              
* Variables:                                                                   
* N/A                                                                       
*                                                                              
* Description:                                                                 
* This function will run one animation on the LED grid continuously until a cup
* gets removed from the table. Once a pod is removed, it will run a new animation,
* in this case End_Blast(side), to coincide with the cup being removed. Once a
* cup is removed, the current animation will be halted and End_Blast(side) will                                                                     
* be called which will shoot a line across the LED grid to whichever side that the
* cup was removed from, it will then do a small 'exploding' effect when it 'hits'
* the other side. Once the animation finishes, the previous animation will pick up
* where it left off.
*
* In this case, the 'detected pod removal' animation that I chose to use is
* End_Blast(side) and the original animation is Pong_Game().
*******************************************************************************/
void Animate_On_Detection(void)
{
  INT8 pod = 0;
  static UINT8 tracker = 0;
  
  //If a previous 'cup removal detected' animation is not running check the 
  //pod detection states
  if (tracker == 0)
  {
    //If a pods detection state has been modified it will be saved in 'pod'
    //Otherwise 'pod' will equal 0, indicating no change
    pod = On_Pod_Change();
    
    //If 'pod' is a negative integer it means that the cup has been removed.
    //If the absolute value of 'pod' is between 1 & 10, the removed cup was
    //on the MASTER SIDE. Start the 'detected pod removal' animation.
    if ((pod < 0) && (abs(pod) < 11))
      tracker = End_Blast(MASTER_SIDE);
    
    //If 'pod' is a negative integer it means that the cup has been removed.
    //If the absolute value of 'pod' is between 11 & 20, the removed cup was
    //on the SECONDARY SIDE. Start the 'detected pod removal' animation.
       else if ((pod < 0) && (abs(pod) < 21))
      tracker = End_Blast(SECONDARY_SIDE);
    
    //There was no change in the pods detection states, continue original animation  
    else
    	Pong_Game();
  }
  
  //A 'detected cup removal' animation has not finished yet. Allow it to finish
  else
    tracker = End_Blast(tracker);     
}  
	
/*******************************************************************************
* Function: Ball_Washer(UINT8 bw, UINT16 fan_speed, UINT16 pump_speed)                                                                    
*                                                                              
* Variables:                                                                   
* bw -> The ball washer that is to be updated (#1 or #2)                                                                            
* fan_speed -> The speed of the fan                                                                             
* pump_speed -> The speed of the water pump                                                                       
*                                                                              
* Description:                                                                 
* This function controls the speed of the fan and water pump for both ball washers.
* The fan and pump motors are connected to outputs 12, 13, 14 and 15 on the 5th
* TLC5940 chip, which also controls LED rings #1 - #12. Therefore we can use the
* same functions that control the LED rings to control the ball washers.                                                                             
*******************************************************************************/
void Ball_Washer(UINT8 bw, UINT16 fan_speed, UINT16 pump_speed)
{
  //Update the fan and water pump speeds for Ball Washer #1
	if (bw == 1)
	{
		LED_no(13,fan_speed);
		LED_no(14,pump_speed);
	}
	
	//Update the fan and water pump speeds for Ball Washer #2
	else
	{
		LED_no(15,fan_speed);
		LED_no(16,pump_speed);
	}			
}	

/*******************************************************************************
* Function: Ball_Washer_Jam_Error(UINT8 error_code)                                                                 
*                                                                              
* Variables:                                                                   
* error_code -> Contains the error code to determine which BW is jammed
*                                 
* Description:                                                                 
* This function will take an error code and check to see if it matches an error
* code for eith of the two ball washers. If there is an error, the function will
* blink the LED rings that pertain to the jammed bal washer 5x times to alert
* the user that they our jammed. The ball washer jam error code can be reset by
* inserting your fingers into the entry and exit holes of the jammed ball washer.
* This function will see that both sensors are blocked and re-enable the ball
* washer and remove the error flag.                                                                          
*******************************************************************************/
void Ball_Washer_Jam_Error(UINT8 error_code)
{
  UINT8 delay = 38; //Delay for ~300ms
  UINT8 bw_sensors;
  UINT8 bw_compare;
  static UINT32 tmark = 0;
  static UINT8 last_seq = 0xFF;
	
  //Set variables to start up state if seq[x] is in reset state
  if (seq[17] == 0xFF)
  {
    seq[17] = 0;
    last_seq = 0xFF;    
  }
  
  //If the seq has changed, update the ring that corresponds to the sequence
  if (seq[17] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[17];
    tmark = count32;
    
    //Shift the ball washer bits 20 places to the right so that their bits are the
    //first four bits in the UINT8 variable
    bw_sensors = (sensor_bits >> 20) & (BW1_ENTRY | BW1_EXIT);
    
    //OR the two sensor values together, we will compare the IR bit sensor values
    //against this to see if both of these sensors are being blocked
    bw_compare = (BW1_ENTRY | BW1_EXIT);
    
    //Check to see if the user is resetting the error flag by blocking the
    //entry and exit sensors of the ball washer at the same time
    if (bw_sensors == bw_compare)
    {
      //Reset flags to allow the ball washer to function again
      DIAGNOSE_ERROR = 0;
      error_code = 0x00;
      BW_ACTIVE = 0;
      BW1_JAM = 0;
      
      //Reset seq[17] and exit the function
      seq[17] = 0xFF;
      return;
    } 
    
    //Shift the ball washer bits 20 places to the right so that their bits are the
    //first four bits in the UINT8 variable
    bw_sensors = (sensor_bits >> 20) & (BW2_ENTRY | BW2_EXIT);
    
    //OR the two sensor values together, we will compare the IR bit sensor values
    //against this to see if both of these sensors are being blocked
    bw_compare = (BW2_ENTRY | BW2_EXIT);
    
    //Check to see if the user is resetting the error flag by blocking the
    //entry and exit sensors of the ball washer at the same time
    if (bw_sensors == bw_compare)
    {
      //Reset flags to allow the ball washer to function again
      DIAGNOSE_ERROR = 0;
      error_code = 0x00;
      BW_ACTIVE = 0;
      BW2_JAM = 0;
      
      //Reset seq[17] and exit the function
      seq[17] = 0xFF;
      return;
    }  
	  
	  //Seeing as we are using the ball washer LED rings in this function we will
	  //set this flag so no other functions will write to them    
    BW_ACTIVE = 1;
    
    //If ball washer #1 is jammed, blink the entry and exit sensor five times 
    if (error_code == BW1_JAM_ERROR_CODE)
    {
      //Every even loop turn the entry and exit rings on
      if (seq[17] % 2)
      {
        LED_no(9,4095);
        LED_no(10,4095);
      }
      
      //Every odd loop turn the entry and exit rings off
      else
      {
        LED_no(9,0);
        LED_no(10,0);
      }
    }
    
    //If ball washer #2 is jammed blink the entry and exit sensor five times 
    else if (error_code == BW2_JAM_ERROR_CODE)
    {
      //Every even loop turn the entry and exit rings on
      if (seq[17] %2)
      {
        LED_no(11,4095);
        LED_no(12,4095);
      }
      
      //Every odd loop turn the entry and exit rings off
      else
      {
        LED_no(11,0);
        LED_no(12,0);
      }
    }        
  }  
      
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[17]++;
  
  //If all sequences have been performed, reset seq[x] and clear the flags
  if (seq[17] > 10)
  {   
    DIAGNOSE_ERROR = 0;
    BW_ACTIVE = 0;
    seq[17] = 0xFF;
    return;
  }  
}  
  
/*******************************************************************************
* Function: Ball_Washers_Detect(UINT8 detection)                                                            
*                                                                              
* Variables:     
* detection -> This variable should contain the sensor_detect data from Check_Sensors()                                                               
*                                                                              
* Description:  
* This function will take in an 8-bit variable (detection) which contains the 
* object detection data for the ball washers. The bit layout of the 'detection'
* is below this paragraph. This is to be used in conjunction with the return
* value from the function Check_Sensors(). The bits must be shifted 20 places to
* the left so that we mask off the pod sensor bits and only use the ball washer
* object detection bits. Here is an example:
*
*	sensors = Check_Sensors();
*	Ball_Washers_Detect(sensors>>20);
*
* bit 0 - Ball Washer #1 Entry Sensor
* bit 1 - Ball Washer #1 Exit Sensor
* bit 2 - Ball Washer #2 Entry Sensor
* bit 3 - Ball Washer #2 Exit Sensor
*
*******************************************************************************/
void Ball_Washers_Detect(UINT8 detection)
{    
  static UINT8 tracker = 0;		
  static UINT8 repeats = 0;
  static UINT16 delay = 0;
  static UINT32 tmark = 0;
	
	//This switch statement will keep all of the sequences of the ball washers
	//in the proper order through the variable 'tracker'
	switch (tracker)
	{
  	//Check to see if either ball washer has detected a ball. Only one ball washer can 
  	//be used at a time and Ball Washer #1 takes precedence if a ball is dropped into
  	//each ball washer at the same time. The reason we only use one ball washer at a time
  	//is to ensure that we don't overheat the ULN2803A driver by overloading it.
  	case 0:
  	        //Check to see if Ball Washer #1 has detected a ball
            if (detection & BW1_ENTRY)
            { 
              //If a ball is stuck set an error flag and update the error code
              if (BW1_JAM)
              {
	              //Reset the repeat value
	              repeats = 0;
	              
                //Update the error code and set an error flag
                error_code = BW1_JAM_ERROR_CODE;
                DIAGNOSE_ERROR = 1;
                return;
              }  
              
          	  //Set this bit so that other functions know that the ball washer is active
          	  BW_ACTIVE = 1;
          	  
          	  //If the ball washer LED rings are fading in an animation, stop them
          	  ring_update &= 0x0FF;
          	  
          	  // Turn off the ball washer LED rings in case they are already on
          	  LED_no(9,0);
          	  LED_no(10,0);
          	  LED_no(11,0);
          	  LED_no(12,0);
          	  
              //Turn on the the entry LED ring and start the water pump
          	  Fade_Ring(9,LEDring[8],4095,120);
          	  Ball_Washer(BW1,BW1_FAN_OFF,BW1_PUMP_SPEED);
          	  
          	  //Update tracker and set a delay of 1.76ms (220 * 8ms per count32 increment)
          	  //Update tmark as we will be using Time_Check() to provide the delay
          	  tracker = 1;
          	  delay = 220;
          	  tmark = count32;
            }
	   	      
	   	      //Check to see if Ball Washer #2 has detected a ball
            else if (detection & BW2_ENTRY)
            { 
              //If a ball is stuck set an error flag and update the error code
              if (BW2_JAM)
              {
	              //Reset the repeat value
	              repeats = 0;
	              
                //Update the error code and set an error flag
                error_code = BW2_JAM_ERROR_CODE;
                DIAGNOSE_ERROR = 1;
                return;
              } 
              
          	  //Set this bit so that other functions know that the ball washer is active
          	  BW_ACTIVE = 1;
          	  
          	  //If the ball washer LED rings are fading in an animation, stop them
          	  ring_update &= 0x0FF;
          	 
          	  // Turn off the ball washer LED rings in case they are already on
          	  LED_no(9,0);
          	  LED_no(10,0);
          	  LED_no(11,0);
          	  LED_no(12,0);
          	  
              //Turn on the the entry LED ring and start the water pump
          	  Fade_Ring(11,LEDring[10],4095,120);
          	  Ball_Washer(BW2,BW2_FAN_OFF,BW2_PUMP_SPEED);
          	  
          	  //Update tracker and set a delay of 1.76ms (220 * 8ms per count32 increment)
          	  //Update tmark as we will be using Time_Check() to provide the delay
          	  tracker = 5;
          	  delay = 200;
          	  tmark = count32;
            }
	   	      break;      
    
    //Wait until 1.76s has passed then turn off the water pump and enable the blower fan      
    case 1:
           if (Time_Check(&tmark,delay))
           {
             //Enable the blower fan
          	  Ball_Washer(BW1,BW1_FAN_SPEED,BW1_PUMP_OFF); 
          	  
          	  //This controls the timeout delay in case the ball gets stuck in the BW
          	  //The delay is set for 2.0s. The timeout delay in the next statement will 
          	  //loop BW_REPEAT amount of times before timing out.
          	  delay = 250;
          	  
          	  //Continue to next statement
          	  tracker = 2;
           }  
	   	     break;      
             
  	case 2:
  	        //Check to see if the ball has passed the exit sensor
            if (detection & BW1_EXIT)
            {	
              //Turn off the entry LED ring and turn on the exit LED ring   	
              Fade_Ring(9,LEDring[8],0,80);
	   	        Fade_Ring(10,LEDring[9],4095,120);
	   	        
          	  //Update tracker and set a delay of 2.92s (365 * 8ms per count32 increment)
          	  //Update tmark as we will be using Time_Check() to provide the delay   
          	  tracker = 3;
          	  delay = 365;
          	  tmark = count32;
            }  
            
           //If the ball does not pass the exit sensor within (2.0s * BW_REPEATS)seconds
           //turn off the LED ring, shut off the blower fan and set a flag
           if (Time_Check(&tmark,delay))
           { 
             repeats++;
                         	 
             //If this timed out it means that the ball never blew out of the pipe
             //and passed the exit sensor. Set a flag and disable the ball washer.
             if (repeats >= BW_REPEATS)
             {
               //Shut off the LED ring and the blower fan	
               Fade_Ring(9,LEDring[8],0,80);
            	 Ball_Washer(BW1,BW1_FAN_OFF,BW1_PUMP_OFF); 
            	 
            	 //Set this flag so that the ball washer is disabled since it timed out
  	   	       BW1_JAM = 1;
  	   	       
  	   	       //Reset these to their startup values
  	   	       repeats = 0;
  	   	       tracker = 0;
  	   	      
            	 //The ball washer is no longer active, clear this bit
            	 BW_ACTIVE = 0;
             }  
           }     
	   	     break;           
	   	      
	  //Wait until 2.92s has passed and then start fading the exit LED ring and slow the
    //blower fan speed down. Set delay equal to 640ms and update tracker.
    case 3:
           if (Time_Check(&tmark,delay))
           {
             //Turn off the EXIT LED ring
  	         Fade_Ring(10,LEDring[9],0,60);
  	         
  	         //Slow down the ball washer fan. If the ping pong ball has not been
  	         //grabbed yet it will fall back down the pipe and trip the EXIT sensor
	   	       Ball_Washer(BW1,BW1_FAN_LOW_SPEED,BW1_PUMP_OFF); 
	   	       
	   	       //Update tracker and set a delay of 640ms
	   	       tracker = 4;
           	 delay = 80;
          	 tmark = count32;
           }  
	   	      break;  

    //Poll the EXIT sensor for 640ms, if the sensor detects the ping pong ball turn
    //the fan back on at its regular speed so the ball blows back up the pipe, turn
    //the LED ring back on, set the next delay for 2.0s and set the tracker 
    //back to 3. This will loop (n_amount_of_times < BW_REPEATS) before it gives up
    //and turns off the ball washer (setting BW1_JAM) or else the player grabs the 
    //ball before it times out.
    case 4:
           //Check to see if the ball has fallen back down the pipe and passed the 
           //exit sensor. If this has repeated too many times, allow this statement
           //to time out and shut off the ball washer.
           if ((detection & BW1_EXIT) && (repeats < BW_REPEATS))
           {	   	
             //Turn on the exit LED ring 
	   	       Fade_Ring(10,LEDring[9],4095,60);
	   	       
	   	       //Turn the fan speed back up so that the ball will blow up the pipe 
          	 Ball_Washer(BW1,BW1_FAN_SPEED,BW1_PUMP_OFF); 
          	  
          	 //Update tracker and set a delay of 2.0s (250 * 8ms per count32 increment)
          	 //Set tracker to 3 so this will loop back to the last case statement  
          	 tracker = 3;
          	 delay = 250;
          	 tmark = count32;
          	 
          	 //Increment repeats, if the player never grabs the ping pong ball from the
          	 //ball washer we do NOT want it to run indefinitely. 
          	 repeats++;
           }          
           
           //The operation has either timed out or the player grabbed the ball from
           //the ball washer. Turn off the LED ring and shut off the blower fan. 
           if (Time_Check(&tmark,delay))
           { 
             //Shut off the LED ring and the blower fan
	   	       Fade_Ring(12,LEDring[11],0,120);
          	 Ball_Washer(BW1,BW1_FAN_OFF,BW1_PUMP_OFF); 
          	 
          	 //If this timed out it means that the ball fell back down in the pipe,
          	 //set a flag so that we can warn the players that a ball is stuck.
	   	       if (repeats >= BW_REPEATS)
	   	         BW1_JAM = 1;
	   	       
	   	       //Reset these to their startup values
	   	       repeats = 0;
	   	       tracker = 0;
	   	      
          	 //The ball washer is no longer active, clear this bit
          	 BW_ACTIVE = 0;
            }  
	   	      break; 

    //Wait until 1.76ms has passed then turn off the water pump and enable the blower fan                     
    case 5:
           if (Time_Check(&tmark,delay))
           {
             //Enable the blower fan
          	  Ball_Washer(BW2,BW2_FAN_SPEED,BW2_PUMP_OFF); 
          	  
          	  //This controls the timeout delay in case the ball gets stuck in the BW
          	  //The delay is set for 2.0s. The timeout delay in the next statement will 
          	  //loop BW_REPEAT amount of times before timing out.
          	  delay = 250;
          	  
          	  //Continue to next statement
          	  tracker = 6;
           }  
	   	     break;      
             
  	case 6:
  	       //Check to see if the ball has passed the exit sensor
           if (detection & BW2_EXIT)
           {	   	
              //Turn off the entry LED ring and turn on the exit LED ring   
             Fade_Ring(11,LEDring[10],0,80);
	   	       Fade_Ring(12,LEDring[11],4095,120);
	   	        
          	 //Update tracker and set a delay of 2.92s (365 * 8ms per count32 increment)
          	 //Update tmark as we will be using Time_Check() to provide the delay   
          	 tracker = 7;
          	 delay = 365;
          	 tmark = count32;
           }  
            
           //If the ball does not pass the exit sensor within (2.0s * BW_REPEATS)seconds
           //turn off the LED ring, shut off the blower fan and set a flag
           if (Time_Check(&tmark,delay))
           { 
             repeats++;
                         	 
             //If this timed out it means that the ball never blew out of the pipe
             //and passed the exit sensor. Set a flag and disable the ball washer.
             if (repeats >= BW_REPEATS)
             {
               //Shut off the LED ring and the blower fan	
               Fade_Ring(10,LEDring[9],0,80);
            	 Ball_Washer(BW2,BW2_FAN_OFF,BW2_PUMP_OFF); 
            	 
            	 //Set this flag so that the ball washer is disabled
  	   	       BW2_JAM = 1;
  	   	       
  	   	       //Reset these to their startup values
  	   	       repeats = 0;
  	   	       tracker = 0;
  	   	      
            	 //The ball washer is no longer active, clear this bit
            	 BW_ACTIVE = 0;
             }  
           }   
	   	     break;        
          
    //Wait until 2.92s has passed and then start fading the exit LED ring and slow the
    //blower fan speed down. Set delay equal to 640ms and update tracker.
    case 7:
           if (Time_Check(&tmark,delay))
           {
             //Turn off the EXIT LED ring
  	         Fade_Ring(12,LEDring[11],0,60);
  	         
  	         //Slow down the ball washer fan. If the ping pong ball has not been
  	         //grabbed yet it will fall back down the pipe and trip the EXIT sensor
	   	       Ball_Washer(BW2,BW2_FAN_LOW_SPEED,BW2_PUMP_OFF);  
	   	       
	   	       //Update tracker and set a delay of 640ms
	   	       tracker = 8;
           	 delay = 80;
          	 tmark = count32;
           }  
	   	      break;        
          
    //Poll the EXIT sensor for 640ms, if the sensor detects the ping pong ball turn
    //the fan back on at its regular speed so the ball blows back up the pipe, turn
    //the LED ring back on, set the next delay for 2.0s and set the tracker 
    //back to 7. This will loop (n_amount_of_times < BW_REPEATS) before it gives up
    //and turns off the ball washer (setting BW2_JAM) or else the player grabs the 
    //ball before it times out.
    case 8:
           //Check to see if the ball has fallen back down the pipe and passed the 
           //exit sensor. If this has repeated too many times, allow this statement
           //to time out and shut off the ball washer.
           if ((detection & BW2_EXIT) && (repeats < BW_REPEATS))
           {	   	
             //Turn on the exit LED ring 
	   	       Fade_Ring(12,LEDring[11],4095,60);
	   	       
	   	       //Turn the fan speed back up so that the ball will blow up the pipe 
          	 Ball_Washer(BW2,BW2_FAN_SPEED,BW2_PUMP_OFF); 
          	  
          	 //Update tracker and set a delay of 2.0s (250 * 8ms per count32 increment)
          	 //Set tracker to 7 so this will loop back to the last case statement  
          	 tracker = 7;
          	 delay = 250;
          	 tmark = count32;
          	 
          	 //Increment repeats, if the player never grabs the ping pong ball from the
          	 //ball washer we do NOT want it to run indefinitely. 
          	 repeats++;
           }          
           
           //The operation has either timed out or the player grabbed the ball from
           //the ball washer. Turn off the LED ring and shut off the blower fan. 
           if (Time_Check(&tmark,delay))
           { 
             //Shut off the LED ring and the blower fan
	   	       Fade_Ring(12,LEDring[11],0,120);
          	 Ball_Washer(BW2,BW2_FAN_OFF,BW2_PUMP_OFF); 
          	 
          	 //If this timed out it means that the ball fell back down in the pipe,
          	 //set a flag so that we can warn the players that a ball is stuck.
	   	       if (repeats >= BW_REPEATS)
	   	         BW2_JAM = 1;
	   	       
	   	       //Reset these to their startup values
	   	       repeats = 0;
	   	       tracker = 0;
	   	      
          	 //The ball washer is no longer active, clear this bit
          	 BW_ACTIVE = 0;
            }  
	   	      break; 
	  
	  //This should never execute, if tracker gets changed to a value not listed, reset it to 0 	      
	  default: tracker = 0; 
	           break;                   
  } 
}	

/*******************************************************************************
* Function: UINT8 Color_Throb(RGB outside_color, RGB inside_color)                                                                 
*                                                                              
* Variables:                                                                   
* outside_color -> This determines the color of the RGB pods on the outside                                                                         
* inside_color -> This determines the color of RGB pods #5 and #15
*                                                                              
* Description:                                                                 
* This function will set all of the RGB pods on each side to one color except
* for the center RGB pod. There will be one outside color that flashes around
* each pod in a triangle fashion and the center will fade in and out with the
* color determined by inside_color.  
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                          
*******************************************************************************/
UINT8 Color_Throb(RGB outside_color, RGB inside_color)
{  
	static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
  int fade_rate = 12;
  int delay = 15;
  
  //The triangular order of the RGB pods
  UINT8 order[10] = {1,2,4,7,8,9,10,6,3};
	
	//The dimmed outside color
	RGB dim_color; 
	
	//Calculate the dimmed color value for the outside pods
	dim_color.red = outside_color.red / 32;
	dim_color.green = outside_color.green / 32;
	dim_color.blue = outside_color.blue / 32;
	
  //If this flag is cleared, the function is just beginning. Set all variables
  //back to default values.
  if (seq[10] == 0xFF)
  {
    seq[10] = 0;
    last_seq = 0xFF;    
  }
  
  //If the seq has changed, update the ring that corresponds to the sequence
  if (seq[10] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[10];
    tmark = count32;
    
    //Start circulating around the triangle of the RGB pods and lighting
    //each one up full brightness at a time before dimming it again
    if (seq[10] == 0)
    {   
      Fade_To(3,PODn[2],dim_color,fade_rate); 
      Fade_To(13,PODn[12],dim_color,fade_rate);   
      Fade_To(1,PODn[0],outside_color,fade_rate);    
      Fade_To(11,PODn[10],outside_color,fade_rate);   
    }  
    
    //Light up the next RGB pod with full brightness while dimming the last 
    //one (which would have been full brightness).
    else
    {
      Fade_To(order[seq[10]-1],PODn[order[seq[10]-1]-1],dim_color,fade_rate);   
      Fade_To(order[seq[10]-1]+10,PODn[order[seq[10]-1]+9],dim_color,fade_rate); 
        
      Fade_To(order[seq[10]],PODn[order[seq[10]]-1],outside_color,fade_rate);  
      Fade_To(order[seq[10]]+10,PODn[order[seq[10]]+9],outside_color,fade_rate);   
    }  
    
    //Handle the center pods separately from the rest of the pods
    if (seq[10] == 4)
    {
      Fade_To(5,PODn[4],inside_color,50);  
      Fade_To(15,PODn[14],inside_color,50);  
	  }
	  
	  else if (seq[10] == 8)
	  {
    	//Calculate the dimmed color value for the inside pods
    	dim_color.red = inside_color.red / 32;
    	dim_color.green = inside_color.green / 32;
    	dim_color.blue = inside_color.blue / 32;
	    
	    //Set RGB pods #5 and #15 with their dimmed color
      Fade_To(5,PODn[4],dim_color,50);
      Fade_To(15,PODn[14],dim_color,50);  
   }   		  
  }  
      
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[10]++;
  
  //If all sequences have been performed, reset seq[10] and return a 0
  if (seq[10] > 8)
  {
    seq[10] = 0xFF;
    return 0;
  }  	
  
  //If the animation hasn't completed return a 1
  return 1;
}   
  
/*******************************************************************************
* Function: Crossfade_Rings(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will cycle through the LED rings fading them in and out starting
* at opposite ends from each other.   
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                          
*******************************************************************************/
UINT8 Crossfade_Rings(void)
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
  int delay = 60; 
  
  //If this flag is cleared, the function is just beginning. Set all variables
  //back to default values.
  if (seq[18] == 0xFF)
  {
	  seq[18] = 0;
    last_seq = 0xFF; 
  }  
  
  //If the seq has changed, update the next set of rings that are on opposite
  //sides of the table from each other
  if (seq[18] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[18];
    tmark = count32;
	 
	  //Use the seq[18] to determine which set of rings to update
	  switch (seq[18])
		{
			  case 0:  
  						 //If neither of the ball washers are active update their LED rings
						   if ((BW_ACTIVE == 0))
							{
								Fade_Ring(9,LEDring[8],RING_MAX,RING_FADE_RATE);	
								Fade_Ring(11,LEDring[10],RING_MAX,RING_FADE_RATE);
							 }
							 
							 //Ball washer LED rings are in use, continue to the next sequence
							 else
							  seq[18]++;
							  
								 break;
								 		
			  case 1:  
  						 //If neither of the ball washers are active update their LED rings
						   if ((BW_ACTIVE == 0))
							{
								Fade_Ring(10,LEDring[9],RING_MAX,RING_FADE_RATE);	
								Fade_Ring(12,LEDring[11],RING_MAX,RING_FADE_RATE);
							 }
							 
							 //Ball washer LED rings are in use, continue to the next sequence
							 else
							  seq[18]++;
							  
								 break;			 
			  			 
			  case 2: Fade_Ring(1,LEDring[0],RING_MAX,RING_FADE_RATE);	
			  			  Fade_Ring(8,LEDring[7],RING_MAX,RING_FADE_RATE); 
			  			  break;
			  			 	
			  case 3: Fade_Ring(4,LEDring[3],RING_MAX,RING_FADE_RATE); 	
		 				    Fade_Ring(5,LEDring[4],RING_MAX,RING_FADE_RATE); 
		 				    break;	
		 				   
			  case 4: Fade_Ring(2,LEDring[1],RING_MAX,RING_FADE_RATE); 	
						    Fade_Ring(7,LEDring[6],RING_MAX,RING_FADE_RATE); 
						    break;
				
			  case 5: Fade_Ring(3,LEDring[2],RING_MAX,RING_FADE_RATE); 	
						 	  Fade_Ring(6,LEDring[5],RING_MAX,RING_FADE_RATE); 
						 	  break;
		  
			  case 6:  
  						 //If neither of the ball washers are active update their LED rings
						   if ((BW_ACTIVE == 0))
							{
								Fade_Ring(9,LEDring[8],RING_MIN,RING_FADE_RATE);	
								Fade_Ring(11,LEDring[10],RING_MIN,RING_FADE_RATE);
							 }
							 
							 //Ball washer LED rings are in use, continue to the next sequence
							 else
							  seq[18]++;
							  
								 break;
								 		
			  case 7:  
  						 //If neither of the ball washers are active update their LED rings
						   if ((BW_ACTIVE == 0))
							{
								Fade_Ring(10,LEDring[9],RING_MIN,RING_FADE_RATE);	
								Fade_Ring(12,LEDring[11],RING_MIN,RING_FADE_RATE);
							 }
							 
							 //Ball washer LED rings are in use, continue to the next sequence
							 else
							  seq[18]++;
							  
								 break;		 
			  			 
			  case 8: Fade_Ring(1,LEDring[0],RING_MIN,RING_FADE_RATE);	
			  			  Fade_Ring(8,LEDring[7],RING_MIN,RING_FADE_RATE); 
			  			  break;
			  			 	
			  case 9: Fade_Ring(4,LEDring[3],RING_MIN,RING_FADE_RATE); 	
		 				    Fade_Ring(5,LEDring[4],RING_MIN,RING_FADE_RATE); 
		 				    break;	
		 				   
			  case 10: Fade_Ring(2,LEDring[1],RING_MIN,RING_FADE_RATE); 	
						     Fade_Ring(7,LEDring[6],RING_MIN,RING_FADE_RATE); 
						     break;
				
			  case 11: Fade_Ring(3,LEDring[2],RING_MIN,RING_FADE_RATE); 	
						 	   Fade_Ring(6,LEDring[5],RING_MIN,RING_FADE_RATE); 
						 	   break;
		}							   
  }
  
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[18]++;
  
  //If all sequences have been performed, reset seq[18] to its default value 
  //and return a 0
  if (seq[18] > 11)
  {
    seq[18] = 0xFF;  
    return 0;
  }
  
  //The animation has not completed yet, return a 1
  return 1;  
}

/*******************************************************************************
* Function: Cycle_Colors(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will cycle each of the pods through each of the default 10 colors.
* It is great to use for testing the RGB LEDs on each pod and to ensure that none
* are faulty. 
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                            
*******************************************************************************/
UINT8 Cycle_Colors(void)
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0; 
  UINT16 delay = 720;
  UINT8 i;
    
  //If this flag is cleared, the function is just beginning. Set all variables
  //back to default values.
  if (seq[1] == 0xFF)
  {
    seq[1] = 0;
    last_seq = 0xFF;  
  }    
  
  //If the seq has changed, update the cycle and change colors
  if (seq[1] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[1];
    tmark = count32;
    
    //Update pods to the next color defined in RGB COLOR (Globals.h)
		for (i = 0;i < 20;i++)
		  Fade_To(i+1,PODn[i],COLOR[seq[1]+1],50);    
  }
  
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[1]++; 
    
  //If all sequences have been performed, reset seq[1] to its default state and
  //return a 0 indicating the end of the animation
  if (seq[1] > 10)
  {
    seq[1] = 0xFF; 
    return 0;
  }
  
  //If the animation has not finished all of its sequences, return a 1
  return 1;  
}
  
/*******************************************************************************
* Function: Cycle_Combined_Animations(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will cycle through 10 different animations. More can be added
* by adjusting the code. 
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                             
*******************************************************************************/
UINT8 Cycle_Combined_Animations(void)
{
  static UINT16 delay = 300;
  static UINT32 tmark = 0;
  static UINT8 last_seq = 0xFF;
	
  //Set variables to start up state if seq[x] is in reset state
  if (seq[15] == 0xFF)
  {
    seq[15] = 0;
    last_seq = 0xFF;    
  }
  
  //If the seq has changed, update the ring that corresponds to the sequence
  if (seq[15] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[15];
    tmark = count32;  
  }
    
  switch (seq[15])
  { 
    case 0:  
            delay = TIME_DELAY_5S;
            break;
  }  
     
      
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[15]++;
  
  //If all sequences have been performed, reset all sequences
  //and return a 0 to indicate that the routine is finished
  if (seq[15] > 9)
  {
    Reset_Sequences(ALL_SEQUENCES);
    return 0;
  }  
  
  //The function has not finished
  return 1;
}  
  
/*******************************************************************************
* Function: Cycle_Grid_Animations(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will cycle through 10 different animations. More can be added
* by adjusting the code. 
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                                
*******************************************************************************/
UINT8 Cycle_Grid_Animations(void)
{
  static INT16 delay = 300;
  static UINT32 tmark = 0;
  static UINT8 last_seq = 0xFF;
	
  //Set variables to start up state if seq[x] is in reset state
  if (seq[12] == 0xFF)
  {
    seq[12] = 0;
    last_seq = 0xFF;    
  }
  
  //If the seq has changed, update the ring that corresponds to the sequence
  if (seq[12] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[12];
    tmark = count32;  
    
    //If the sequence is updated, clear the grid
    Clear_Grid(); 
  }
  
  //This function will cycle through all of the animations below.
  //More animations can be added by adding a case statement and
  //adjusting the amount of sequences to loop through below this statment.  
  switch (seq[12])
  { 
    case 0: Draw_Sine(REGULAR);  
            delay = TIME_DELAY_5S;
            break;
            
    case 1: Draw_Sine(INVERTED);  
            delay = TIME_DELAY_5S;
            break;
            
    case 2: Checkers();  
            delay = TIME_DELAY_5S;
            break;
   
    //Here we set the delay to a very large value as we don't want the
    //interrupt delay check to move on to the next animation until we 
    //know that all of the text is off of the grid.        
    case 3: 
    				Set_Scrolling_Text("HELLO INSTRUCTABLES!");
    				
    				//If text has finished scrolling, reset scroll flag and set
    				//delay equal to 1 so that the Time_Check(a,b) below advances to 
    				//the next animation	
    				if (SCROLL_FINISHED)
    				{
    				 delay = 1;
    				 SCROLL_FINISHED = 0;
    				} 
    				
    				//If the scroll has not finished yet, set delay to a high value
    				//so that Time_Check(a,b) does not continue to the next animation
    			  else
            delay = TIME_DELAY_5M;  
            
            break;
    
    //Much like the scrolling text, wait until the animation has completed
    //before allowing the function to continue to the next animation        
    case 4: 
            //If the pong animation has completed, allow Time_Check(a,b) to interrupt
            if (Pong_Game() == 0)    			
              delay = 1;
            
            //The animation is not over. Make sure that Time_Check(a,b) does not
            //update the seq[x] variable
            else   
            delay = TIME_DELAY_4M;
            
            break;
            
    case 5: LED_Scoreboard(sensor_bits);  
            delay = TIME_DELAY_5S;
            break;
            
    case 6: End_Blast(MASTER_SIDE);  
            delay = TIME_DELAY_5S;
            break;
            
    case 7: Dual_Wave(REGULAR);  
            delay = TIME_DELAY_10S;
            break;
            
    case 8: Corner_Circles();  
            delay = TIME_DELAY_10S;
            break;
            
    case 9: Box_Grid_In();  
            delay = TIME_DELAY_10S;
            break;
  }  
    
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[12]++;
  
  //If all sequences have been performed, reset all sequences
  //and return a 0 to indicate that the routine is finished
  if (seq[12] > 9)
  {
    //Reset the seq[x] variables that are used by the grid animations
    Reset_Sequences(GRID_SEQUENCES);
    return 0;
  }  
  
  //The function has not finished
  return 1;
}  
  
/*******************************************************************************
* Function: Cycle_Pod_Animations(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will cycle through 10 different animations. More can be added
* by adjusting the code. 
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                      
*******************************************************************************/
UINT8 Cycle_Pod_Animations(void)
{
  static UINT16 delay = 300;
  static UINT32 tmark = 0;
  static UINT8 last_seq = 0xFF;
	
  //Set variables to start up state if seq[x] is in reset state
  if (seq[13] == 0xFF)
  {
    seq[13] = 0;
    last_seq = 0xFF;    
  }
  
  //If the seq has changed, update the ring that corresponds to the sequence
  if (seq[13] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[13];
    tmark = count32;  
  }
    
  //This function will cycle through all of the animations below.
  //More animations can be added by adding a case statement and
  //adjusting the amount of sequences to loop through below this statment.  
  switch (seq[13])
  { 
    case 0: Fade_Cups(); 
            delay = TIME_DELAY_10S;
            break;
  
    case 1: Color_Throb(COLOR[RED],COLOR[BLUE]);
            delay = TIME_DELAY_10S;
            break;
  
    case 2: Ripple_Out(50,80);
            delay = TIME_DELAY_10S;
            break;
            
    case 3: Pyramid_Chase();
            delay = TIME_DELAY_10S;
            break;
  
    case 4: Cycle_Colors();
            delay = TIME_DELAY_10S;
            break;
            
    case 5: Pod_Detect(sensor_bits,COLOR[PINK],COLOR[GREEN]);
            delay = TIME_DELAY_5S;
            break;
  }  
     
      
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[13]++;
  
  //If all sequences have been performed, reset all sequences
  //and return a 0 to indicate that the routine is finished
  if (seq[13] > 5)
  {
    //Reset the seq[x] variables that are used by the pod animations
    Reset_Sequences(POD_SEQUENCES);
    return 0;
  }  
  
  //The function has not finished
  return 1;
}  
  
/*******************************************************************************
* Function: Cycle_Ring_Animations(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will cycle through 10 different animations. More can be added
* by adjusting the code. 
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                              
*******************************************************************************/
UINT8 Cycle_Ring_Animations(void)
{
  static UINT16 delay = 300;
  static UINT32 tmark = 0;
  static UINT8 last_seq = 0xFF;
	
  //Set variables to start up state if seq[x] is in reset state
  if (seq[14] == 0xFF)
  {
    seq[14] = 0;
    last_seq = 0xFF;    
  }
  
  //If the seq has changed, update the ring that corresponds to the sequence
  if (seq[14] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[14];
    tmark = count32;  
  }
    
  //This function will cycle through all of the animations below.
  //More animations can be added by adding a case statement and
  //adjusting the amount of sequences to loop through below this statment. 
  switch (seq[14])
  { 
    case 0: 
    				Cycle_Rings(); 
            delay = TIME_DELAY_10S;
            break;
            
    case 1: 
    				Ring_Chase(); 
            delay = TIME_DELAY_10S;
            break;
            
    case 2: 
    				Crossfade_Rings(); 
            delay = TIME_DELAY_15S;
            break;
  }  
     
      
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[14]++;
  
  //If all sequences have been performed, reset all sequences
  //and return a 0 to indicate that the routine is finished
  if (seq[14] > 2)
  {
    //Reset the seq[x] variables that are used by the ring animations
    Reset_Sequences(LED_RING_SEQUENCES);
    return 0;
  }  
  
  //The function has not finished
  return 1;
} 

/*******************************************************************************
* Function: Cycle_Rings(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will cycle through the LED rings fading them in and out. It is
* a good test function to ensure that all rings are working. 
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                           
*******************************************************************************/
UINT8 Cycle_Rings(void)
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
  int delay = 20; 
  
  //If this flag is cleared, the function is just beginning. Set all variables
  //back to default values.
  if (seq[2] == 0xFF)
  {
	  seq[2] = 0;
    last_seq = 0xFF; 
  }  
  
  //If the seq has changed, update the ring that corresponds to the sequence
  if (seq[2] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[2];
    tmark = count32;
	 
	  //Use the seq[2] to determine which set of rings to update
	  switch (seq[2])
		{
			  case 0:  
  						 //If neither of the ball washers are active update their LED rings
						   if ((BW_ACTIVE == 0))
							{
								Fade_Ring(9,LEDring[8],RING_MAX,RING_FADE_RATE);	
								Fade_Ring(10,LEDring[9],RING_MAX,RING_FADE_RATE);
							}
							 
							 //Ball washer LED rings are in use, continue to the next sequence
							 else
							  seq[2]++;
							  
								 break;			 
			  			 
			case 1:  Fade_Ring(1,LEDring[0],RING_MAX,RING_FADE_RATE);	
			  			 Fade_Ring(5,LEDring[4],RING_MAX,RING_FADE_RATE); break;
			  			 	
			case 2:  Fade_Ring(2,LEDring[1],RING_MAX,RING_FADE_RATE); 	
		 				   Fade_Ring(6,LEDring[5],RING_MAX,RING_FADE_RATE); break;	
		 				   
			case 3:  Fade_Ring(3,LEDring[2],RING_MAX,RING_FADE_RATE); 	
						   Fade_Ring(7,LEDring[6],RING_MAX,RING_FADE_RATE); break;
				
			case 4:  Fade_Ring(4,LEDring[3],RING_MAX,RING_FADE_RATE); 	
						 	 Fade_Ring(8,LEDring[7],RING_MAX,RING_FADE_RATE); break;
							
		 	case 5:  
  						 //If neither of the ball washers are active update their LED rings
							 if ((BW_ACTIVE == 0))
							 {		 	
								 Fade_Ring(11,LEDring[10],RING_MAX,RING_FADE_RATE);	
		  			     Fade_Ring(12,LEDring[11],RING_MAX,RING_FADE_RATE); 
		  			   }  
							 
							 //Ball washer LED rings are in use, continue to the next sequence
							 else
							  seq[2]++;
							     
		  			   break;	
			  			 
		 	case 6:  
  						 //If neither of the ball washers are active update their LED rings
							 if ((BW_ACTIVE == 0))
							 {  
								 Fade_Ring(9,LEDring[8],RING_MIN,RING_FADE_RATE);	
			  		     Fade_Ring(10,LEDring[9],RING_MIN,RING_FADE_RATE); 
			  	     }
							 
							 //Ball washer LED rings are in use, continue to the next sequence
							 else
							  seq[2]++;
							  
			  		   break;
			  		      
			case 7:  Fade_Ring(1,LEDring[0],RING_MIN,RING_FADE_RATE); 	
							 Fade_Ring(5,LEDring[4],RING_MIN,RING_FADE_RATE); break;
				
			case 8:  Fade_Ring(2,LEDring[1],RING_MIN,RING_FADE_RATE); 	
							 Fade_Ring(6,LEDring[5],RING_MIN,RING_FADE_RATE); break;
				
			case 9:  Fade_Ring(3,LEDring[2],RING_MIN,RING_FADE_RATE);	
							 Fade_Ring(7,LEDring[6],RING_MIN,RING_FADE_RATE); break;
				
			case 10: Fade_Ring(4,LEDring[3],RING_MIN,RING_FADE_RATE); 	
							 Fade_Ring(8,LEDring[7],RING_MIN,RING_FADE_RATE); break;
							
		  case 11:  
  					   //If neither of the ball washers are active update their LED rings
						   if ((BW_ACTIVE == 0))
					 	   { 
								 Fade_Ring(11,LEDring[10],RING_MIN,RING_FADE_RATE);	
		   			     Fade_Ring(12,LEDring[11],RING_MIN,RING_FADE_RATE); 
		   			   } 
							 
							 //Ball washer LED rings are in use, continue to the next sequence
							 else
							  seq[2]++;
							    
		   			   break;		
			  			 						
		}		   
  }
  
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[2]++;
  
  //If all sequences have been performed, reset seq[2] to its default value 
  //and return a 0
  if (seq[2] > 11)
  {
    seq[2] = 0xFF;  
    return 0;
  }
  
  //The animation has not completed yet, return a 1
  return 1;  
}

/*******************************************************************************
* Function: Display_Error(UINT8 error)                                                                   
*                                                                              
* Variables:                                                                   
* error -> The error code used to diagnose what is going wrong
*                                 
* Description:                                                                 
* This function will display the specified error on the table.                                                                           
*******************************************************************************/
void Display_Error(UINT8 error)
{
  //Execute the specified errors function
  switch (error)
  {
    case BW1_JAM_ERROR_CODE: Ball_Washer_Jam_Error(error); break;
    case BW2_JAM_ERROR_CODE: Ball_Washer_Jam_Error(error);  break;
  }      
}  
	
/*******************************************************************************
* Function: Dual_Wave_On_Detection(void)                                                             
*                                                                              
* Variables:                                                                   
* N/A                                                                       
*                                                                              
* Description:                                                                 
* This function will run one animation on the LED grid continuously until a cup
* gets removed from the table. Once a pod is removed, it will run a new animation,
* in this case End_Blast(side), to coincide with the cup being removed. Once a
* cup is removed, the current animation will be halted and End_Blast(side) will                                                                     
* be called which will shoot a line across the LED grid to whichever side that the
* cup was removed from, it will then do a small 'exploding' effect when it 'hits'
* the other side. Once the animation finishes, the previous animation will pick up
* where it left off.
*
* In this case, the 'detected pod removal' animation that I chose to use is
* End_Blast(side) and the original animation is Dual_Wave(cnt).
*******************************************************************************/
void Dual_Wave_On_Detection(void)
{
  INT8 pod = 0;
  static UINT8 tracker = 0;
  static UINT8 cnt = 0;
  
  //If a'cup removal detected' animation is not running check the 
  //pod detection states
  if (tracker == 0)
  {
    //If a pods detection state has been modified it will be saved in 'pod'
    //Otherwise 'pod' will equal 0, indicating no change
    pod = On_Pod_Change();
    
    //If 'pod' is a negative integer it means that the cup has been removed.
    //If the absolute value of 'pod' is between 1 & 10, the removed cup was
    //on the MASTER SIDE. Start the 'detected pod removal' animation.
    if ((pod < 0) && (abs(pod) < 11))
    {
      //Run the 'detected cup removal' animation and save its return value
      //as that is how we will know when it has finished
      tracker = End_Blast(MASTER_SIDE);
      
      //Update 'cnt' to switch to the next Sine wave view
      cnt++;
    }  
    
    //If 'pod' is a negative integer it means that the cup has been removed.
    //If the absolute value of 'pod' is between 11 & 20, the removed cup was
    //on the SECONDARY SIDE. Start the 'detected pod removal' animation.
    else if ((pod < 0) && (abs(pod) < 21))
    {
      //Run the 'detected cup removal' animation and save its return value
      //as that is how we will know when it has finished
      tracker = End_Blast(SECONDARY_SIDE);
        
      //Update 'cnt' to switch to the next Sine wave view
      cnt++;
    }    
    
    //There was no change in the pods detection states, continue original animation  
    else
    {
      //Flip flop the value of count each cup removal, this way we can switch
      //between a regular and inverted animation every time a cup is removed
	    if ((cnt % 2)==0)
	    	cnt = 0;
	    	
	    else 
	     cnt = 1;
	    
	    //Draw a regular or inverted sine wave (depends on value of cnt)
    	Dual_Wave(cnt);
    }	
  }
  
  //A 'detected cup removal' animation has not finished yet. Allow it to finish
  else
    tracker = End_Blast(tracker);     
} 
	
/*******************************************************************************
* Function: Fade_All_Pods(RGB NEW, UINT16 fade_rate)                                                               
*                                                                              
* Variables:                                                                                                                                         
* NEW -> An RGB struct which holds the color that the pod is fading to                                                                            
* fade_rate -> The amount of delay between each fade step                                                                          
*                                                                              
* Description:                                                                 
* This function will fade all of the RGB pods to one specific color over a
* specified fade rate.                                                                       
*******************************************************************************/
void Fade_All_Pods(RGB NEW, UINT16 fade_rate)                   
{
  UINT8 i;
  
  //Set each pod to fade to the specified color
  for (i = 0;i < 20;i++)
    Fade_To(i+1,PODn[i],NEW,fade_rate);
}
	
/*******************************************************************************
* Function: Fade_All_Rings(UINT16 duty_cycle, UINT16 fade_rate, UINT8 mode)                                                                
*                                                                              
* Variables:                                                                   
* duty_cycle -> The 12-bit PWM duty cycle (0 - 4095)
* fade_rate -> Determines the amount of time it takes to complete the fade
* mode -> Determines whether it will adjust the ball washer LED rings                                                                          
*                                                                              
* Description:                                                                 
* This function will adjust all of the LED rings PWM duty cycle to the value of
* 'duty_cycle'. If mode is equal to zero, this function will not modify the ball
* washer LED rings and if mode is a non-zero value it will adjust the ball washer
* rings unless the BW_ACTIVE flag is set. If the variable 'mode' is set to 0, 
* the ball washer LED rings will not be modified. If it is set to 1, all LED rings
* on the table will be modified unless a ball washer is in use.                                                                       
*******************************************************************************/
void Fade_All_Rings(UINT16 duty_cycle, UINT16 fade_rate, UINT8 mode)                   
{
  UINT8 i;
  UINT8 amount;
  
  //If mode is set to a non-zero value and BW_ACTIVE is cleared, we will adjust
  //all of the LED rings
  if ((mode != 0) && (BW_ACTIVE == 0))
    amount = 12;
 
  //Only adjust the 8x main LED rings on the rails   
  else
    amount = 8;
  
  //Update the LED rings  
  for (i = 0;i < amount;i++)
    Fade_Ring(i+1,LEDring[i],duty_cycle,fade_rate);
} 

/*******************************************************************************
* Function: Fade_Cups(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will fade the pods in and out the colors red, green, blue and                                                                             
* white. Out of the 4x rows of pods on each side, each row will be a different                                                                             
* color and they will continually fade through each color.   
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                           
*******************************************************************************/
UINT8 Fade_Cups(void)
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
  int fade_rate = 60;
  int delay = 130;
  
  //If this flag is cleared, the function is just beginning. Set all variables
  //back to default values.
  if (seq[4] == 0xFF)
  {
    seq[4] = 0;
    last_seq = 0xFF;
  }  
  
  //This function will rotate through 4 different sequences and color patterns
  //If the seq has changed, update the ring that corresponds to the sequence
  if (seq[4] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[4];
    tmark = count32;
      
    switch (seq[4])
    {
      case 0: Fade_To(20,PODn[19],COLOR[BLUE],fade_rate);      
              Fade_To(19,PODn[18],COLOR[BLUE],fade_rate);      
              Fade_To(18,PODn[17],COLOR[BLUE],fade_rate);      
              Fade_To(17,PODn[16],COLOR[BLUE],fade_rate);      
              Fade_To(16,PODn[15],COLOR[RED],fade_rate);       
              Fade_To(15,PODn[14],COLOR[RED],fade_rate);       
              Fade_To(14,PODn[13],COLOR[RED],fade_rate);       
              Fade_To(13,PODn[12],COLOR[GREEN],fade_rate);     
              Fade_To(12,PODn[11],COLOR[GREEN],fade_rate);     
              Fade_To(11,PODn[10],COLOR[WHITE],fade_rate);    
              Fade_To(10,PODn[9],COLOR[BLUE],fade_rate);    
              Fade_To(9,PODn[8],COLOR[BLUE],fade_rate);    
              Fade_To(8,PODn[7],COLOR[BLUE],fade_rate);    
              Fade_To(7,PODn[6],COLOR[BLUE],fade_rate);    
              Fade_To(6,PODn[5],COLOR[RED],fade_rate);     
              Fade_To(5,PODn[4],COLOR[RED],fade_rate);     
              Fade_To(4,PODn[3],COLOR[RED],fade_rate);     
              Fade_To(3,PODn[2],COLOR[GREEN],fade_rate);   
              Fade_To(2,PODn[1],COLOR[GREEN],fade_rate);   
              Fade_To(1,PODn[0],COLOR[WHITE],fade_rate);   
              break;
              
      case 1: Fade_To(20,PODn[19],COLOR[RED],fade_rate);       
              Fade_To(19,PODn[18],COLOR[RED],fade_rate);       
              Fade_To(18,PODn[17],COLOR[RED],fade_rate);       
              Fade_To(17,PODn[16],COLOR[RED],fade_rate);       
              Fade_To(16,PODn[15],COLOR[GREEN],fade_rate);     
              Fade_To(15,PODn[14],COLOR[GREEN],fade_rate);     
              Fade_To(14,PODn[13],COLOR[GREEN],fade_rate);     
              Fade_To(13,PODn[12],COLOR[WHITE],fade_rate);     
              Fade_To(12,PODn[11],COLOR[WHITE],fade_rate);     
              Fade_To(11,PODn[10],COLOR[BLUE],fade_rate);     
              Fade_To(10,PODn[9],COLOR[RED],fade_rate);     
              Fade_To(9,PODn[8],COLOR[RED],fade_rate);     
              Fade_To(8,PODn[7],COLOR[RED],fade_rate);     
              Fade_To(7,PODn[6],COLOR[RED],fade_rate);     
              Fade_To(6,PODn[5],COLOR[GREEN],fade_rate);   
              Fade_To(5,PODn[4],COLOR[GREEN],fade_rate);   
              Fade_To(4,PODn[3],COLOR[GREEN],fade_rate);   
              Fade_To(3,PODn[2],COLOR[WHITE],fade_rate);   
              Fade_To(2,PODn[1],COLOR[WHITE],fade_rate);   
              Fade_To(1,PODn[0],COLOR[BLUE],fade_rate);    
              break;    
    
      case 2: Fade_To(20,PODn[19],COLOR[GREEN],fade_rate);     
              Fade_To(19,PODn[18],COLOR[GREEN],fade_rate);     
              Fade_To(18,PODn[17],COLOR[GREEN],fade_rate);     
              Fade_To(17,PODn[16],COLOR[GREEN],fade_rate);     
              Fade_To(16,PODn[15],COLOR[WHITE],fade_rate);     
              Fade_To(15,PODn[14],COLOR[WHITE],fade_rate);     
              Fade_To(14,PODn[13],COLOR[WHITE],fade_rate);     
              Fade_To(13,PODn[12],COLOR[BLUE],fade_rate);      
              Fade_To(12,PODn[11],COLOR[BLUE],fade_rate);      
              Fade_To(11,PODn[10],COLOR[RED],fade_rate);      
              Fade_To(10,PODn[9],COLOR[GREEN],fade_rate);   
              Fade_To(9,PODn[8],COLOR[GREEN],fade_rate);   
              Fade_To(8,PODn[7],COLOR[GREEN],fade_rate);   
              Fade_To(7,PODn[6],COLOR[GREEN],fade_rate);   
              Fade_To(6,PODn[5],COLOR[WHITE],fade_rate);   
              Fade_To(5,PODn[4],COLOR[WHITE],fade_rate);   
              Fade_To(4,PODn[3],COLOR[WHITE],fade_rate);   
              Fade_To(3,PODn[2],COLOR[BLUE],fade_rate);    
              Fade_To(2,PODn[1],COLOR[BLUE],fade_rate);    
              Fade_To(1,PODn[0],COLOR[RED],fade_rate);                     
              break; 
    
      case 3: Fade_To(20,PODn[19],COLOR[WHITE],fade_rate);     
              Fade_To(19,PODn[18],COLOR[WHITE],fade_rate);     
              Fade_To(18,PODn[17],COLOR[WHITE],fade_rate);     
              Fade_To(17,PODn[16],COLOR[WHITE],fade_rate);     
              Fade_To(16,PODn[15],COLOR[BLUE],fade_rate);      
              Fade_To(15,PODn[14],COLOR[BLUE],fade_rate);      
              Fade_To(14,PODn[13],COLOR[BLUE],fade_rate);      
              Fade_To(13,PODn[12],COLOR[RED],fade_rate);       
              Fade_To(12,PODn[11],COLOR[RED],fade_rate);       
              Fade_To(11,PODn[10],COLOR[GREEN],fade_rate);    
              Fade_To(10,PODn[9],COLOR[WHITE],fade_rate);   
              Fade_To(9,PODn[8],COLOR[WHITE],fade_rate);   
              Fade_To(8,PODn[7],COLOR[WHITE],fade_rate);   
              Fade_To(7,PODn[6],COLOR[WHITE],fade_rate);   
              Fade_To(6,PODn[5],COLOR[BLUE],fade_rate);    
              Fade_To(5,PODn[4],COLOR[BLUE],fade_rate);    
              Fade_To(4,PODn[3],COLOR[BLUE],fade_rate);    
              Fade_To(3,PODn[2],COLOR[RED],fade_rate);     
              Fade_To(2,PODn[1],COLOR[RED],fade_rate);     
              Fade_To(1,PODn[0],COLOR[GREEN],fade_rate);   
              break;    
    }          
  }
    
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[4]++;
  
  //If all sequences have been performed, reset seq[4] and return a 0
  if (seq[4] > 3)
  {
    seq[4] = 0xFF;
    return 0;
  }  
  
  //If the animation has not completed return a 1
  return 1;
}

/*******************************************************************************
* Function: Fade_Out_Table(UINT16 fade_rate)                                                                
*                                                                              
* Variables:                                                                   
* fade_rate -> The rate at which the features will fade out at                                                                          
*                                                                              
* Description:                                                                 
* This function isn't very useful. I used it at the very end of my final video
* to fade out the table.                                                                             
*******************************************************************************/
void Fade_Out_Table(UINT16 fade_rate)
{
  int i;
  UINT16 delay;
  
  //Make sure that all LED rings are turned on so that we can fade them out
  Fade_All_Rings(601,1,ALL_LED_RINGS);
  Delay_ms(10);
  
  //Begin fading out the RGB pods
  Fade_All_Pods(COLOR[BLACK],fade_rate);
  
  //Begin fading out the LED rings
  Fade_All_Rings(0,fade_rate*2,ALL_LED_RINGS);
  
  //Calculate the delay rate for the grid
  delay = (fade_rate / 3); 
  
  //Fade out the grid at the same rate as the LED rings and RGB pods  
  for (i = 15; i >= 0;i--)
  {
    HT1632_Set_PWM(i);
    Delay_ms(delay);
  }  
  
  //Clear the LED grid
  Clear_Grid();
  HT1632_UPDATE = 1; 
  
  //Reset the grid to max brightness
  HT1632_Set_PWM(15);	  
}  
	
/*******************************************************************************
* Function: Fade_Ring(UINT8 LED, float old, float new, UINT16 delay)                                                                    
*                                                                              
* Variables:                                                                   
* LED -> The # of the LED ring that is to be updated (1 - 12)                                                                            
* old -> The brightness that the LED ring will be fading from                                                                            
* new -> The brightness that the LED ring will be fading to                                                                            
* delay -> The amount of delay between each fade step                                                                            
*                                                                              
* Description:                                                                 
* This function allows the user to fade the LED rings in and out at different
* brightnesses with different speeds.                                                                             
*******************************************************************************/
void Fade_Ring(UINT8 LED, float old, float new, UINT16 delay)
{ 
  //Calculate the fade step value
  ring_diff[LED-1].value = (float)((float)new - (float)old) / delay;
  
  //Set the brightness of the LED ring that it is fading from
  LED_no(LED,(UINT16)old); 
  ring_diff[LED-1].delay = delay; 
  
  //Set the flag bit that corresponds the LED ring that is to be updated 
  ring_update |= ((UINT16)1 << (UINT16)(LED-1));
} 

/*******************************************************************************
* Function: Fade_State(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function keeps track of the state of the RGB pods and the LED rings.                                                                              
* The TIMER3 interrupt calls this function to see if any pods or LED rings need                                                                             
* to be updated from a fade routine.                                                                             
*******************************************************************************/
void Fade_State(void)
{
  UINT8 i;

  //If no flags are set for either the pods or the rings, return the function
  if ((pod_update | ring_update) == 0)
    return;

  //Loop through specified pods, if a flag is set for that pod, update its values
	for (i = 0;i < 20;i++)
	{
	  if ((pod_update >> i) & 0x01)
	  {
  	  //Adjust the pod one fade step
	    PODn[i].red += RGB_diff[i].red;
	    PODn[i].green += RGB_diff[i].green;
	    PODn[i].blue += RGB_diff[i].blue;   
	    RGB_diff[i].delay--;
	      
	    //Update the pod with the new values
	    RGB_Pod(i+1,PODn[i].red,PODn[i].green,PODn[i].blue);
	      
	    //If the delay is over, clear the pods update flag
	    if (!RGB_diff[i].delay)
	       pod_update &= (~((UINT32)1 << i)); 
	  }
	}  
	
	//Check LED rings. If a flag is set, update the values
	for (i = 0;i < 12;i++)
	{
	  if ((ring_update >> i) & 0x01)
	  {
  	  //Adjust the ring one fade step
		  LEDring[i] += ring_diff[i].value; 
	    ring_diff[i].delay--;
	    
	    //Update the LED ring
	    LED_no(i+1, LEDring[i]);
	    
	    //If the delay is over, clear the rings update flag
	    if (!ring_diff[i].delay)
	       ring_update &= (~(1 << i)); 
	  }
	}  
}

/*******************************************************************************
* Function: Fade_To(UINT8 pod, RGB OLD, RGB NEW, UINT16 delay)                                                                     
*                                                                              
* Variables:                                                                   
* pod -> The pod # that is to be updated                                                                            
* OLD -> An RGB struct which holds the color that the pod is fading from                                                                            
* NEW -> An RGB struct which holds the color that the pod is fading to                                                                            
* delay -> The amount of delay between each fade step                                                                        
*                                                                              
* Description:                                                                 
* This function will fade the specified RGB pod from the one color to another                                                                             
* over an amount of time specified by 'delay'. The time interval of 'delay' is                                                                          
* determined by the time period of the TIMER3 interrupt.
*******************************************************************************/
void Fade_To(UINT8 pod, RGB OLD, RGB NEW, UINT16 delay)
{ 
  //Calculate the fade step value for RED, GREEN and BLUE values
  RGB_diff[pod-1].red = (float)(NEW.red - OLD.red) / delay;
  RGB_diff[pod-1].green = (float)(NEW.green - OLD.green) / delay;
  RGB_diff[pod-1].blue = (float)(NEW.blue - OLD.blue) / delay;
  RGB_diff[pod-1].delay = delay; 
  
  //Set the fade from RGB pod color
  RGB_Pod(pod,OLD.red,OLD.green,OLD.blue); 
  
  //Set the flag bit that corresponds the pod # that is to be updated 
  pod_update |= ((UINT32)1 << (UINT32)((UINT32)pod-1));
}

/*******************************************************************************
* Function: LED_no(UINT8 LED, UINT16 data)                                                                    
*                                                                              
* Variables:                                                                   
* LED -> 1 of 16 outputs from the 5th TLC5940 on the Master PCB. This controls 
*        LED rings #1 - #12 and the Ball Washer fan1, pump1, fan2 and pump2 motors.                                                                
* data -> The 24 bytes (192 bits) of data that control the 16x outputs on the TLC5940 
*                                                                            
* Description:                                                                 
* This function is used to update the LED rings and the ball washer motors.
* To see how the data is packed, refer to the chart for the LED rings located in
* the projects 'Charts' folder.
*
* Bytes 0..23 (Each output is 12-bits (4096 steps) which is 1.5 bytes)
*
* Output -> LSb .. MSb
*
* LED01 -> 24.0 - 22.5
* LED02 -> 22.5 - 21.0
* LED03 -> 21.0 - 19.5
* LED04 -> 19.5 - 18.0
* LED05 -> 18.0 - 16.5
* LED06 -> 16.5 - 15.0
* LED07 -> 15.0 - 13.5
* LED08 -> 13.5 - 12.0
* LED09 -> 12.0 - 10.5
* LED10 -> 10.5 - 9.0
* LED11 ->  9.0 - 7.5
* LED12 ->  7.5 - 6.0
* FAN01 ->  6.0 - 4.5
* PUMP1 ->  4.5 - 3.0
* FAN02 ->  3.0 - 1.5
* PUMP2 ->  1.5 - 0.0
*                                                                             
*******************************************************************************/
void LED_no(UINT8 LED, UINT16 data)
{
  UINT8 spot; 
  UINT8 startx[16] = {23,22,20,19,17,16,14,13,11,10,8,7,5,4,2,1};
  
  //Get the location of the bits that are to be updated
  spot = startx[LED-1];
  
  //Update the specified outputs value
  LEDring[LED-1] = data & 0x0FFF;
  
  //If the 1.5 bytes (12 bits) of data start in the middle of a byte in the
  //packed data, update the 4x MSb's then update the next byte in the array
  //
  if ((LED % 2) == 0)
  {  
    RGB_data[spot] &= 0x00F;
    RGB_data[spot] |= ((data << 4) & 0x0F0);
    RGB_data[spot-1] = (data >> 4);
  }
  
  //If the 1.5 bytes (12 bits) of data start at the beginning of a byte in the
  //packed data, update the first 8x LSb's then bitmask in the last 4x MSb's to
  //the next byte in the array
  else
  {
    RGB_data[spot] = 0x0FF & data;
    RGB_data[spot-1] &= 0x0F0;
    RGB_data[spot-1] |= (data >> 8);
  }    
  
  //Prepare TLC5940 interrupt to update the outputs
  WAIT_XLAT = 1;
  TLC5940_Update();
} 

/*******************************************************************************
* Function: On_Pod_Change(void)                                                                  
*                                                                              
* Variables:                                                                   
* N/A
*                                 
* Description:                                                                 
* This function will check the sensor data that is passed into it and return a
* 0 if no pods have changed since the previous run. If a pod has changed, the 
* pods numerical value (1 - 20) will be returned from the function and if the returned
* value is a positive value it means that the cup has been added to the table. If
* the returned value is a negative value, it means that the cup has been removed
* from the table. To eliminate false detections, the cup that is detected must
* be consistent for POLL_AMOUNT # of loops, if the sensed values changed before
* the polling times out it will assume that it was a false detect and discard the
* previous data.                                                                      
*******************************************************************************/
INT8 On_Pod_Change(void)
{
  static UINT32 last_sensor_values = 0x00000000;
  static INT8 saved_pod = 0;
  static UINT16 pod_counter = 0;
  UINT8 i;
  UINT32 change;
  
  //See if a pod has been removed or added to the table
  change = sensor_bits ^ last_sensor_values;
  
  //If the values have changed from the last loop through, reset
  //the variables used for polling the IR sensors
  if (change)
  {
  	pod_counter = 0;
  	saved_pod = 0;
  }	
  
  //If a cup has been removed, return the pods # as a negative integer  
  if (change &  ~sensor_bits)
  {
	  //A pod has been modified, start the polling counter
	  pod_counter = 1;
	  
    //Find which pod has been changed (The lower the pod #, the higher
    //the priority it has).
    for (i = 0;i < 20;i++)
    {
      //Find the pod that has been modified and return the data that will
      //equal the pods number. The value will be a negative integer to 
      //inform us that the cup has been removed from the table.
      if ((change >> i) & 0x00000001)
        saved_pod = (-(i+1));
    }
  }  
  
  //If a pod has been added, return the pods # as a positive integer    
  else if (change & sensor_bits)
  {
	  //A pod has been modified, start the polling counter
	  pod_counter = 1;
	  
    //Find which pod has been changed (The lower the pod #, the higher
    //the priority it has).    
    for (i = 0;i < 20;i++)
    {
      //Find the pod that has been modified and return the data that will
      //equal the pods number. The value will be a positive integer to inform
      //us that the cup has been added from the table.
      if ((change >> i) & 0x00000001)
        saved_pod = (i+1);
    }
  }        
  
  //Update the variable that we use to keep track of the sensor values  
  last_sensor_values = sensor_bits;  
  
  //If the polling has started for a sensor, increment it each time
 	if (pod_counter >= 1)
		pod_counter++;
	
	//If the polling amount has been reached, reset the polling counter and
	//return the value of the pod along with the detection state
  if (pod_counter > POLL_AMOUNT)
  {
	  pod_counter = 0;
	  return saved_pod;
	}

  //No pods have changed, return 0 
  return 0;   
}
  
/*******************************************************************************
* Function: Our_Test_Animation(void)                                                                   
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* I have made this function to provide an example of how I create my interrupt
* delay routines. We will run a simple animation on the LED grid with this. I
* have dedicated the global variable 'seq[24]' for use in this function.    
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                       
*******************************************************************************/
UINT8 Our_Test_Animation(void)
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0; 
  UINT16 delay = 5;
    
  //If seq[24] is in its default start, ready all variables to restart the animation
  if (seq[24] == 0xFF)
  {
    seq[24] = 0;
    last_seq = 0xFF;  
  }    
  
  //If the seq has changed, update the cycle and move onto the next sequence
  if (seq[24] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[24];
    tmark = count32;
    
    ////////////** Animation code goes here **///////////
    
    //Light up all 32 ROWs consecutively
    LED_data[seq[24]] = 0xFFF;  
    HT1632_UPDATE = 1;
    
    /////////////////////////////////////////////////////
  }
  
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[24]++; 
    
  //If all sequences have been performed, reset seq[24] to its default starting
  //State and return a 0 to indicate that the animation has finished.
  if (seq[24] > 31)
  {
    //Reset LED_data[x] values; Clear the grid for the next animation
    Clear_Grid();
    
    seq[24] = 0xFF; 
    return 0;
  }
  
  //If the animation hasn't finished return a 1
  return 1;  
} 

/*******************************************************************************
* Function: Pod_Detect(UINT32 detection, RGB off_color, RGB on_color)                                                               
*                                                                              
* Variables:                                                                   
* detection -> This variable should contain the sensor_detect data from Check_Sensors()                                                                            
* off_color -> The color that you want the RGB pod to be when no cup is detected                                                                             
* on_color -> The color that you want the RGB pod to be when a cup is detected                                                                                
*                                                                              
* Description:                                            
* This function will adjust the color of each RGB pod. The pods color will be set
* to 'off_color' if no cup is detected and 'on_color' if a cup is detected.                                                                        
*******************************************************************************/
void Pod_Detect(UINT32 detection, RGB off_color, RGB on_color)
{
  UINT8 i;
  
  //Cycle through all 20 pods
  for (i = 0;i < 20;i++)
  {
    //If a cup is detected on the specified pod then set the pod to on_color
    if ((detection >> i) & 0x00000001)
      Fade_To(i+1,PODn[i],on_color,POD_DETECT_FADERATE);
      
    //If a cup is not detected on the specified pod then set the pod to off_color
    else
      Fade_To(i+1,PODn[i],off_color,POD_DETECT_FADERATE);
  }    
}  

/*******************************************************************************
* Function: Pod_Set_Color(UINT8 pod, RGB color)                                                                    
*                                                                              
* Variables:                                                                   
* pod -> The pod that is to be modified
* color -> An RGB struct that stores the color of the pod                                                                         
*                                                                              
* Description:                                                                 
* This function simply allows the user to update a pod with an RGB structure instead
* of putting the red, green and blue values in manually.                                                                             
*******************************************************************************/
void Pod_Set_Color(UINT8 pod, RGB pod_color)
{
  //Modify the RGB colors of the pod to achieve the color specified
  RGB_Pod(pod,pod_color.red,pod_color.green,pod_color.blue);
}  

/*******************************************************************************
* Function: Pyramid_Chase(void)                                                                  
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will slowly fade the RGB pods through different colors starting
* from the top of the pyramid to the bottom of the pyramid.      
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                         
*******************************************************************************/
UINT8 Pyramid_Chase(void)
{  
	static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
	static UINT8 i = 1;
  int fade_rate = 70;
  int delay = 30;
	
  //Check to see if the animation is just starting/restarting and set variables
  //to start-up values
  if (seq[9] == 0xFF)
  {
    seq[9] = 0;
    last_seq = 0xFF;    
  }
  
  //If the seq has changed, update the new sequence
  if (seq[9] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[9];
    tmark = count32;
    
    //Update the corresponding pods on each side of the table
    Fade_To(seq[9]+1,PODn[seq[9]],COLOR[i],fade_rate); 
    Fade_To(seq[9]+11,PODn[seq[9]+10],COLOR[i],fade_rate); 
  }  
      
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[9]++;
  
  //If all sequences have been performed, reset the variables and increment
  //'i' so that a new color will be used on the next run
  if (seq[9] > 9)
  {
	  i++;
    seq[9] = 0xFF;
    
    //If 'i' is greater than 10 reset it to 1
    i = (i > 10) ? 1 : i;
    
    //Once one complete sequence has completed return a 0
    return 0;
  }  	
  
  //The function hasn't completed a full sequence yet
  return 1;
}  
 
/*******************************************************************************
* Function: Reset_All_Variables(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will reset global variables and features of the table into their
* default states.                                                                             
*******************************************************************************/
void Reset_All_Variables(void)
{
	UINT16 i;
	
	//Clear grid
	Clear_Grid();
	
	//Reset animation sequences
	Reset_Sequences(ALL_SEQUENCES);
	
	//Reset error code
	error_code = 0;
	
	//Reset flags
	FLAG1.all = 0x0000;
	pod_update = 0;
	ring_update = 0;
	
	//Turn off all LED rings and ball washers
	for(i = 1;i < 17;i++)
		LED_no(i,0);		
	
	//Turn off all 20x RGB pods	and reset all fade values
  for (i = 0;i < 20;i++)
  {
	  PODn[i].red = 0;
	  PODn[i].green = 0;
	  PODn[i].blue = 0;
	  
	  RGB_diff[i].delay = 0;
	  
	  RGB_Pod(i+1,PODn[i].red,PODn[i].green,PODn[i].blue);
	} 
}
 
/*******************************************************************************
* Function: Reset_All_Sequences(UINT8 mode)                                                                    
*                                                                              
* Variables:                                                                   
* mode -> Determines which seq[x] variables should be set to their default values                                                                          
*                                                                              
* Description:                                                                 
* This function will reset all of the seq[] variables used for animations back to 
* their default state of 0xFF.                                                                     
*******************************************************************************/
void Reset_Sequences(UINT8 mode)
{
	UINT16 i;
	
	//Refer to Globals.h to see which seq[x] variable belongs to which function
	switch (mode)
	{
  	//Reset all seq[] variables to their default state
  	case ALL_SEQUENCES:
  	
        	  for (i = 0;i < SEQ_AMOUNT;i++)
          		 seq[i] = 0xFF;
          	
            break;
            
    //Reset all seq[x] variables that are used for LED grid animations        
    case GRID_SEQUENCES:
            seq[0]  = 0xFF;
            seq[3]  = 0xFF;
            seq[5]  = 0xFF;
            seq[6]  = 0xFF;
            seq[11] = 0xFF;
            seq[12] = 0xFF;
            seq[16] = 0xFF;
            seq[20] = 0xFF;
            seq[21] = 0xFF;
            seq[24] = 0xFF;
            break;
            
    //Reset all seq[x] variables that are used for RGB pod animations      
    case POD_SEQUENCES:
            seq[1]  = 0xFF;
            seq[4]  = 0xFF;
            seq[8]  = 0xFF;
            seq[9]  = 0xFF;
            seq[10] = 0xFF;
            seq[13] = 0xFF;
            break;
            
    //Reset all seq[x] variables that are used for LED Ring animations      
    case LED_RING_SEQUENCES:
            seq[2]  = 0xFF;
            seq[14]  = 0xFF;
            seq[18]  = 0xFF;
            seq[19]  = 0xFF;
            break;
  }           
}

/*******************************************************************************
* Function: RGB_Pod(UINT8 pod, UINT16 red, UINT16 green, UINT16 blue)                                                                     
*                                                                              
* Variables:                                                                   
* pod ---> The RGB pod that is to be addressed (Master: #1 - #10 | Secondary: #11 - #20)                                                                           
* red ---> The 12-bit value of red                                                                            
* green -> The 12-bit value of green                                                                            
* blue --> The 12-bit value of blue                                                                     
*                                                                              
* Description:                                                                 
* This function will set the RGB LEDs of a specified pod to the color determined                                                                             
* by the user with the values of red, green and blue.                                        
*******************************************************************************/
void RGB_Pod(UINT8 pod, UINT16 red, UINT16 green, UINT16 blue)
{
  UINT8 chip;
  UINT8 spot;
  UINT8 *index;
  
  //Set pointer to starting location of packed data that controls the RGB pods
  index = &RGB_data;
  
  //Find which TLC5940 chip controls the specfied pod
  chip = (pod - 1) / TLC_CHIPS;
  
  //Find the first output on the TLC5940 that control the specified pods RGB LEDs
  spot = (pod - 1) - (chip * 5);
  
  //Calculate the location of the pods RGB values in the data array
  index  += ((chip * 24) + start[spot]) + 24;
  
  //If the 4.5 bytes (36 bits) of data start at the beginning of a byte in the
  //packed data, update the 4.5 bytes of data from the location at *index.
  if (((pod & 0x01) & (!(chip & 0x01))) | (((!(pod & 0x01)) & (chip & 0x01))))
  {
    *index = red & 0x0FF;
    *(--index) = (green << 4) | (red >> 8);
    *(--index) = green >> 4;
    *(--index) = blue & 0xFF;
    *index--;
    *(index) = (*index & 0xF0) | (blue  >> 8);
	}
  
  //If the 4.5 bytes (36 bits) of data start in the middle of a byte in the
  //packed data, update the 4x MSb's of *index then update the next 4 bytes 
  else
  {
    *index = (*index & 0x0F) | ((red & 0x0F) << 4);
    *(--index) = red >> 4;
    *(--index) = green & 0xFF;
    *(--index) = (green >> 8) | (blue << 4);
    *(--index) = blue >> 4;
  }    
 
  //Prepare TLC5940 interrupt to update pods
  WAIT_XLAT = 1;
  TLC5940_Update();
}      
  
/*******************************************************************************
* Function: Ring_Chase(void)                                                                  
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This animation will light all of the LED rings dimly except for one which will
* be at 'max' brightness. Every sequence the bright LED ring will dim and the next
* LED ring in the square will be lit up to full brightness. This function will not
* modify the ball washer LED rings if BW_ACTIVE is equal to 1.       
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                      
*******************************************************************************/
UINT8 Ring_Chase(void)
{
  static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
  int min = 28;  
  int max = 2000; 
  int delay = 25; 
  int fade_rate = 25;
  int i;
  
  //This is all of the LED rings in a square
  int order[12] = {9,1,2,3,4,12,11,8,7,6,5,10};
  
  //If this flag is cleared, the function is just beginning. Set all variables
  //back to default values.
  if (seq[19] == 0xFF)
  {
	  seq[19] = 0;
    last_seq = 0xFF; 
  }  
  
  //If the seq has changed, update the ring that corresponds to the sequence
  if (seq[19] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[19];
    tmark = count32;
    
	  //Cycle through the rings
	  for (i = 0;i < 12;i++)
	  {
  	  //If none of the ball washers are active, modify them when it is their turn
  	  //Otherwise skip the four ball washer LED rings at locations 0, 5, 6 and 11
		  if (!((BW_ACTIVE == 1) && ((i == 0) || (i == 5) || (i == 6) || (i == 11))))   
	    {
  	    //If it is not the LED rings sequence, make sure that it is dimmed
	  	  if (i != seq[19])
	  	    Fade_Ring(order[i],LEDring[order[i]-1],min,fade_rate);
	  	  
	  	  //If it is this LED rings sequence, put it to 'max' brightness  
	  	  else
	  	    Fade_Ring(order[i],LEDring[order[i]-1],max,fade_rate);
	    }	    
	  }      	    						   
  }
  
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[19]++;
  
  //If all sequences have been performed, reset seq[19] to its default value 
  //and return a 0
  if (seq[19] > 11)
  {
    seq[19] = 0xFF;  
    return 0;
  }
  
  //The animation has not completed yet, return a 1
  return 1;  
}
	
/*******************************************************************************
* Function: Ripple_Out(UINT16 fade_rate, UINT16 delay)                                                                    
*                                                                              
* Variables:                                                                   
* fade_rate -> This adjusts the rate of fade for the RGB pods                                                                            
* delay -> This adjusts the amount of delay between each color                                                                            
*                                                                              
* Description:                                                                 
* This function will set the middle RGB pods (5 & 15) to a color first and
* then the surrounding pods will slowly fade into the same color. The process
* will then repeat with a new color producing a rippling effect.     
*
* This animation must be continually looped through with the main code. If the 
* animation finishes, the function will return a 0. If the animation is still in
* progress it will return a 1.                                                                          
*******************************************************************************/
UINT8 Ripple_Out(UINT16 fade_rate, UINT16 delay)
{  
	static UINT8 last_seq = 0xFF;
  static UINT32 tmark = 0;
	
  //Check to see if the animation is starting/restarting
  if (seq[8] == 0xFF)
  {
    seq[8] = 0;
    last_seq = 0xFF;    
  }
  
  //Check to see if a new sequence has to be updated
  if (seq[8] != last_seq)
  {
    //Update the sequence and 'tmark' which is used for timing
    last_seq = seq[8];
    tmark = count32;
    
    //This statement determines what colors and RGB pods fade at what sequences
    switch (seq[8])
    {
      case 0:  
              Fade_To(5,PODn[4],COLOR[RED],fade_rate);  
              Fade_To(15,PODn[14],COLOR[RED],fade_rate);   
              break;
              
      case 1: 
              Fade_To(20,PODn[19],COLOR[RED],fade_rate);  
              Fade_To(19,PODn[18],COLOR[RED],fade_rate);    
              Fade_To(18,PODn[17],COLOR[RED],fade_rate);    
              Fade_To(17,PODn[16],COLOR[RED],fade_rate);       
              Fade_To(16,PODn[15],COLOR[RED],fade_rate);    
              Fade_To(14,PODn[13],COLOR[RED],fade_rate);     
              Fade_To(13,PODn[12],COLOR[RED],fade_rate);   
              Fade_To(12,PODn[11],COLOR[RED],fade_rate);   
              Fade_To(11,PODn[10],COLOR[RED],fade_rate); 
              Fade_To(10,PODn[9],COLOR[RED],fade_rate);  
              Fade_To(9,PODn[8],COLOR[RED],fade_rate);    
              Fade_To(8,PODn[7],COLOR[RED],fade_rate);    
              Fade_To(7,PODn[6],COLOR[RED],fade_rate);       
              Fade_To(6,PODn[5],COLOR[RED],fade_rate);    
              Fade_To(4,PODn[3],COLOR[RED],fade_rate);     
              Fade_To(3,PODn[2],COLOR[RED],fade_rate);   
              Fade_To(2,PODn[1],COLOR[RED],fade_rate);   
              Fade_To(1,PODn[0],COLOR[RED],fade_rate);   
              break;
              
      case 10:  
              Fade_To(5,PODn[4],COLOR[ORANGE],fade_rate); 
              Fade_To(15,PODn[14],COLOR[ORANGE],fade_rate);   
              break;
              
      case 11: 
              Fade_To(20,PODn[19],COLOR[ORANGE],fade_rate);  
              Fade_To(19,PODn[18],COLOR[ORANGE],fade_rate);    
              Fade_To(18,PODn[17],COLOR[ORANGE],fade_rate);    
              Fade_To(17,PODn[16],COLOR[ORANGE],fade_rate);       
              Fade_To(16,PODn[15],COLOR[ORANGE],fade_rate);    
              Fade_To(14,PODn[13],COLOR[ORANGE],fade_rate);     
              Fade_To(13,PODn[12],COLOR[ORANGE],fade_rate);   
              Fade_To(12,PODn[11],COLOR[ORANGE],fade_rate);   
              Fade_To(11,PODn[10],COLOR[ORANGE],fade_rate);   
              Fade_To(10,PODn[9],COLOR[ORANGE],fade_rate);  
              Fade_To(9,PODn[8],COLOR[ORANGE],fade_rate);    
              Fade_To(8,PODn[7],COLOR[ORANGE],fade_rate);    
              Fade_To(7,PODn[6],COLOR[ORANGE],fade_rate);       
              Fade_To(6,PODn[5],COLOR[ORANGE],fade_rate);    
              Fade_To(4,PODn[3],COLOR[ORANGE],fade_rate);     
              Fade_To(3,PODn[2],COLOR[ORANGE],fade_rate);   
              Fade_To(2,PODn[1],COLOR[ORANGE],fade_rate);   
              Fade_To(1,PODn[0],COLOR[ORANGE],fade_rate);   
              break;
              
      case 12:  
              Fade_To(5,PODn[4],COLOR[CYAN],fade_rate); 
              Fade_To(15,PODn[14],COLOR[CYAN],fade_rate);   
              break;
              
      case 13:        
              Fade_To(20,PODn[19],COLOR[CYAN],fade_rate);  
              Fade_To(19,PODn[18],COLOR[CYAN],fade_rate);    
              Fade_To(18,PODn[17],COLOR[CYAN],fade_rate);    
              Fade_To(17,PODn[16],COLOR[CYAN],fade_rate);       
              Fade_To(16,PODn[15],COLOR[CYAN],fade_rate);    
              Fade_To(14,PODn[13],COLOR[CYAN],fade_rate);     
              Fade_To(13,PODn[12],COLOR[CYAN],fade_rate);   
              Fade_To(12,PODn[11],COLOR[CYAN],fade_rate);   
              Fade_To(11,PODn[10],COLOR[CYAN],fade_rate);   
              Fade_To(10,PODn[9],COLOR[CYAN],fade_rate);  
              Fade_To(10,PODn[9],COLOR[CYAN],fade_rate);  
              Fade_To(9,PODn[8],COLOR[CYAN],fade_rate);    
              Fade_To(8,PODn[7],COLOR[CYAN],fade_rate);    
              Fade_To(7,PODn[6],COLOR[CYAN],fade_rate);       
              Fade_To(6,PODn[5],COLOR[CYAN],fade_rate);    
              Fade_To(4,PODn[3],COLOR[CYAN],fade_rate);     
              Fade_To(3,PODn[2],COLOR[CYAN],fade_rate);   
              Fade_To(2,PODn[1],COLOR[CYAN],fade_rate);   
              Fade_To(1,PODn[0],COLOR[CYAN],fade_rate);   
              break;
              
      case 8:  
              Fade_To(5,PODn[4],COLOR[VIOLET],fade_rate);  
              Fade_To(15,PODn[14],COLOR[VIOLET],fade_rate);   
              break;
              
      case 9:         
              Fade_To(20,PODn[19],COLOR[VIOLET],fade_rate);  
              Fade_To(19,PODn[18],COLOR[VIOLET],fade_rate);    
              Fade_To(18,PODn[17],COLOR[VIOLET],fade_rate);    
              Fade_To(17,PODn[16],COLOR[VIOLET],fade_rate);       
              Fade_To(16,PODn[15],COLOR[VIOLET],fade_rate);    
              Fade_To(14,PODn[13],COLOR[VIOLET],fade_rate);     
              Fade_To(13,PODn[12],COLOR[VIOLET],fade_rate);   
              Fade_To(12,PODn[11],COLOR[VIOLET],fade_rate);   
              Fade_To(11,PODn[10],COLOR[VIOLET],fade_rate);   
              Fade_To(10,PODn[9],COLOR[VIOLET],fade_rate); 
              Fade_To(10,PODn[9],COLOR[VIOLET],fade_rate);  
              Fade_To(9,PODn[8],COLOR[VIOLET],fade_rate);    
              Fade_To(8,PODn[7],COLOR[VIOLET],fade_rate);    
              Fade_To(7,PODn[6],COLOR[VIOLET],fade_rate);       
              Fade_To(6,PODn[5],COLOR[VIOLET],fade_rate);    
              Fade_To(4,PODn[3],COLOR[VIOLET],fade_rate);     
              Fade_To(3,PODn[2],COLOR[VIOLET],fade_rate);   
              Fade_To(2,PODn[1],COLOR[VIOLET],fade_rate);   
              Fade_To(1,PODn[0],COLOR[VIOLET],fade_rate);   
              break;
              
      case 2:  
              Fade_To(5,PODn[4],COLOR[GREEN],fade_rate);   
              Fade_To(15,PODn[14],COLOR[GREEN],fade_rate);   
              break;
              
      case 3:         
              Fade_To(20,PODn[19],COLOR[GREEN],fade_rate);  
              Fade_To(19,PODn[18],COLOR[GREEN],fade_rate);    
              Fade_To(18,PODn[17],COLOR[GREEN],fade_rate);    
              Fade_To(17,PODn[16],COLOR[GREEN],fade_rate);       
              Fade_To(16,PODn[15],COLOR[GREEN],fade_rate);    
              Fade_To(14,PODn[13],COLOR[GREEN],fade_rate);     
              Fade_To(13,PODn[12],COLOR[GREEN],fade_rate);   
              Fade_To(12,PODn[11],COLOR[GREEN],fade_rate);   
              Fade_To(11,PODn[10],COLOR[GREEN],fade_rate);   
              Fade_To(10,PODn[9],COLOR[GREEN],fade_rate); 
              Fade_To(10,PODn[9],COLOR[GREEN],fade_rate);  
              Fade_To(9,PODn[8],COLOR[GREEN],fade_rate);    
              Fade_To(8,PODn[7],COLOR[GREEN],fade_rate);    
              Fade_To(7,PODn[6],COLOR[GREEN],fade_rate);       
              Fade_To(6,PODn[5],COLOR[GREEN],fade_rate);    
              Fade_To(4,PODn[3],COLOR[GREEN],fade_rate);     
              Fade_To(3,PODn[2],COLOR[GREEN],fade_rate);   
              Fade_To(2,PODn[1],COLOR[GREEN],fade_rate);   
              Fade_To(1,PODn[0],COLOR[GREEN],fade_rate);   
              break;
              
      case 6:  
              Fade_To(5,PODn[4],COLOR[WHITE],fade_rate); 
              Fade_To(15,PODn[14],COLOR[WHITE],fade_rate);   
              break;
              
      case 7:         
              Fade_To(20,PODn[19],COLOR[WHITE],fade_rate);  
              Fade_To(19,PODn[18],COLOR[WHITE],fade_rate);    
              Fade_To(18,PODn[17],COLOR[WHITE],fade_rate);    
              Fade_To(17,PODn[16],COLOR[WHITE],fade_rate);       
              Fade_To(16,PODn[15],COLOR[WHITE],fade_rate);    
              Fade_To(14,PODn[13],COLOR[WHITE],fade_rate);     
              Fade_To(13,PODn[12],COLOR[WHITE],fade_rate);   
              Fade_To(12,PODn[11],COLOR[WHITE],fade_rate);   
              Fade_To(11,PODn[10],COLOR[WHITE],fade_rate);   
              Fade_To(10,PODn[9],COLOR[WHITE],fade_rate); 
              Fade_To(10,PODn[9],COLOR[WHITE],fade_rate);  
              Fade_To(9,PODn[8],COLOR[WHITE],fade_rate);    
              Fade_To(8,PODn[7],COLOR[WHITE],fade_rate);    
              Fade_To(7,PODn[6],COLOR[WHITE],fade_rate);    
              Fade_To(6,PODn[5],COLOR[WHITE],fade_rate);     
              Fade_To(4,PODn[3],COLOR[WHITE],fade_rate);     
              Fade_To(3,PODn[2],COLOR[WHITE],fade_rate);   
              Fade_To(2,PODn[1],COLOR[WHITE],fade_rate);   
              Fade_To(1,PODn[0],COLOR[WHITE],fade_rate);   
              break;
              
      case 4:  
              Fade_To(5,PODn[4],COLOR[BLUE],fade_rate);   
              Fade_To(15,PODn[14],COLOR[BLUE],fade_rate);   
              break;
              
      case 5:         
              Fade_To(20,PODn[19],COLOR[BLUE],fade_rate);  
              Fade_To(19,PODn[18],COLOR[BLUE],fade_rate);    
              Fade_To(18,PODn[17],COLOR[BLUE],fade_rate);    
              Fade_To(17,PODn[16],COLOR[BLUE],fade_rate);       
              Fade_To(16,PODn[15],COLOR[BLUE],fade_rate);    
              Fade_To(14,PODn[13],COLOR[BLUE],fade_rate);     
              Fade_To(13,PODn[12],COLOR[BLUE],fade_rate);   
              Fade_To(12,PODn[11],COLOR[BLUE],fade_rate);   
              Fade_To(11,PODn[10],COLOR[BLUE],fade_rate);   
              Fade_To(10,PODn[9],COLOR[BLUE],fade_rate); 
              Fade_To(10,PODn[9],COLOR[BLUE],fade_rate);  
              Fade_To(9,PODn[8],COLOR[BLUE],fade_rate);    
              Fade_To(8,PODn[7],COLOR[BLUE],fade_rate);    
              Fade_To(7,PODn[6],COLOR[BLUE],fade_rate);    
              Fade_To(6,PODn[5],COLOR[BLUE],fade_rate);      
              Fade_To(4,PODn[3],COLOR[BLUE],fade_rate);     
              Fade_To(3,PODn[2],COLOR[BLUE],fade_rate);   
              Fade_To(2,PODn[1],COLOR[BLUE],fade_rate);   
              Fade_To(1,PODn[0],COLOR[BLUE],fade_rate);   
              break;
              
      case 14:  
              Fade_To(5,PODn[4],COLOR[MAGENTA],fade_rate);   
              Fade_To(15,PODn[14],COLOR[MAGENTA],fade_rate);   
              break;
              
      case 15:         
              Fade_To(20,PODn[19],COLOR[MAGENTA],fade_rate);  
              Fade_To(19,PODn[18],COLOR[MAGENTA],fade_rate);    
              Fade_To(18,PODn[17],COLOR[MAGENTA],fade_rate);    
              Fade_To(17,PODn[16],COLOR[MAGENTA],fade_rate);       
              Fade_To(16,PODn[15],COLOR[MAGENTA],fade_rate);    
              Fade_To(14,PODn[13],COLOR[MAGENTA],fade_rate);     
              Fade_To(13,PODn[12],COLOR[MAGENTA],fade_rate);   
              Fade_To(12,PODn[11],COLOR[MAGENTA],fade_rate);   
              Fade_To(11,PODn[10],COLOR[MAGENTA],fade_rate);   
              Fade_To(10,PODn[9],COLOR[MAGENTA],fade_rate); 
              Fade_To(10,PODn[9],COLOR[MAGENTA],fade_rate);  
              Fade_To(9,PODn[8],COLOR[MAGENTA],fade_rate);    
              Fade_To(8,PODn[7],COLOR[MAGENTA],fade_rate);    
              Fade_To(7,PODn[6],COLOR[MAGENTA],fade_rate);    
              Fade_To(6,PODn[5],COLOR[MAGENTA],fade_rate);     
              Fade_To(4,PODn[3],COLOR[MAGENTA],fade_rate);     
              Fade_To(3,PODn[2],COLOR[MAGENTA],fade_rate);   
              Fade_To(2,PODn[1],COLOR[MAGENTA],fade_rate);   
              Fade_To(1,PODn[0],COLOR[MAGENTA],fade_rate);   
              break;
    }
  }  
      
  //If the specified delay has elapsed, continue to the next sequence
  if (Time_Check(&tmark,delay))
    seq[8]++;
  
  //If all sequences have been performed, reset seq[8] and return a 0
  if (seq[8] > 15)
  {
    seq[8] = 0xFF;
    return 0;
  }    
  
  //If the animation hasn't finished, return a 1
  return 1;      	
}	
	
/*******************************************************************************
* Function: Set_All_Pods(RGB color)                                                             
*                                                                              
* Variables:                                                                   
* color -> An RGB struct color value                                                                       
*                                                                              
* Description:                                                                 
* This function will set all twenty of the RGB pods to the same color.                                                                     
*******************************************************************************/
void Set_All_Pods(RGB color)
{
  UINT8 i;
  
  //Set all of the pods to the specified color
  for (i = 0;i < 20;i++)
  	Pod_Set_Color(i+1,color);   
}
	
/*******************************************************************************
* Function: Set_All_Rings(UINT16 duty_cycle, UINT8 mode)                                                           
*                                                                              
* Variables:                                                                   
* duty_cycle -> The 12-bit PWM duty cycle (0 - 4095)
* mode -> Determines whether it will adjust the ball washer LED rings                                                                          
*                                                                              
* Description:                                                                 
* This function will adjust all of the LED rings PWM duty cycle to the value of
* 'duty_cycle'. If mode is equal to zero, this function will not modify the ball
* washer LED rings and if mode is a non-zero value it will adjust the ball washer
* rings unless the BW_ACTIVE flag is set.                                                                         
*******************************************************************************/
void Set_All_Rings(UINT16 duty_cycle, UINT8 mode)
{
  UINT8 i;
  UINT8 amount;
  
  //If mode is set to a non-zero value and BW_ACTIVE is cleared, we will adjust
  //all of the LED rings
  if ((mode != 0) && (BW_ACTIVE == 0))
    amount = 12;
 
  //Only adjust the 8x main LED rings on the rails   
  else
    amount = 8;
  
  //Update the LED rings  
  for (i = 1;i <= amount;i++)
    LED_no(i,duty_cycle);
} 

/*******************************************************************************
* Function: Startup_Animation(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function is only displayed once the beer pong table is turned on. It runs                                                                              
* through a few animations for each of the features on the table before continuing                                                                              
* into the main code.                                                                             
*******************************************************************************/
void Startup_Animation(void)
{
	UINT16 i;
  int fade_rate = 400;
  int delay = 650;
  
  UINT16 max = 1795;
	
	//Set the LED grid to it's lowest brightness and turn on all of the LEDs			
	HT1632_Set_PWM(0);
	Fill_Grid();
	HT1632_UPDATE = 1;
	
	//Loop through all of the 16 brightnesses for the LED grid, ending on the brightest			
	for (i = 0;i < 16;i++)
	{
	 HT1632_Set_PWM(i);
	 Delay_ms(100);
	}	
  
  //Disable the VU meter
  VU_On = 0;
  
  //Start fading in each LED ring in doubles. The 2x rings that will light up
  //for each step will be in exact opposite locations on the table.
  Fade_Ring(9,LEDring[8],max,fade_rate);	
  Fade_Ring(11,LEDring[10],max,fade_rate);
	Delay_ms(delay);	
  
  Fade_Ring(10,LEDring[9],max,fade_rate);	
  Fade_Ring(12,LEDring[11],max,fade_rate);
	Delay_ms(delay);	
	
	Fade_Ring(1,LEDring[0],max,fade_rate); 	
	Fade_Ring(8,LEDring[7],max,fade_rate); 
	Delay_ms(delay);	
	 	
	Fade_Ring(5,LEDring[4],max,fade_rate);
	Fade_Ring(4,LEDring[3],max,fade_rate); 
	Delay_ms(delay);
	
	Fade_Ring(2,LEDring[1],max,fade_rate); 	
	Fade_Ring(7,LEDring[6],max,fade_rate); 
	Delay_ms(delay);
		
	Fade_Ring(3,LEDring[2],max,fade_rate); 
	Fade_Ring(6,LEDring[5],max,fade_rate); 	
	Delay_ms(delay);	
		
	//Fade in a different color for each pod on each side. Opposite pods on each 
	//side of the table will light up together.	
	for (i = 0;i < 10;i++)
	{		
	  Fade_To(i+1,PODn[i],COLOR[i+1],50); 
	  Fade_To(i+11,PODn[i+10],COLOR[i+1],50); 
	  Delay_ms(300);
  } 

	//Reset global variables and functions to default states
	Reset_All_Variables();
}

#endif
