/*******************************************************************************
* Title: Miscellaneous.h                                                       
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                      
*                                                                              
* Description:                                                                 
* This file contains various functions that pertain to the operation of the LED grid,                                                                             
* the RGB pods, the LED rings and the ball washers.                                                                                      
*******************************************************************************/
    
#ifndef MISCELLANEOUS_H
#define MISCELLANEOUS_H    
    
/*************************************************
*               Global Variables                 *
*************************************************/ 
//This will be set if either of the ball washers are active
#define BW_ACTIVE       FLAG1.b3

//Alerts the program to display an error message
#define DIAGNOSE_ERROR  FLAG1.b6

//Either of these will be set if any of the ball washers 'jam' (The ball is stuck)
#define BW1_JAM         FLAG1.b7
#define BW2_JAM         FLAG1.b8

/*************************************************
*                   Constants                    *
*************************************************/
//Reset Sequences
#define ALL_SEQUENCES       0
#define GRID_SEQUENCES      1
#define POD_SEQUENCES       2
#define LED_RING_SEQUENCES  3

//Used in Pod_Detect() to set the faderates of the pods
#define POD_DETECT_FADERATE 40

//Used to adjust all of the LED rings at once. All LED rings
//includes the ball washer LED rings.
#define ALL_LED_RINGS       1
#define MAIN_LED_RINGS      0

//Ball washers #1 and #2
#define BW1                 1
#define BW2                 2

//Ball washer REGULAR pump and fan speeds
#define BW1_PUMP_SPEED      3000
#define BW2_PUMP_SPEED      3000
#define BW1_FAN_SPEED       3100
#define BW2_FAN_SPEED       3400

//Ball washer LOW fan speeds, used to detect
//if the ball has been grabbed from the exit hole.
//If it hasn't, this speed will lower the ball back
//down the hole where it will trip the sensor.
#define BW1_FAN_LOW_SPEED  1700
#define BW2_FAN_LOW_SPEED  1700

//Disable Ball Washer Motors
#define BW1_PUMP_OFF        0
#define BW2_PUMP_OFF        0
#define BW1_FAN_OFF         0
#define BW2_FAN_OFF         0

//The amount of times that the ball washer will try to
//blow a stuck ball out of the ball washer before it
//times out and sets an error flag
#define BW_REPEATS          3

//Ball washer IR sensor bits
#define BW1_ENTRY           0x01
#define BW1_EXIT            0x02
#define BW2_ENTRY           0x04
#define BW2_EXIT            0x08

//Ball Washer Error Codes
#define BW1_JAM_ERROR_CODE  0xFE
#define BW2_JAM_ERROR_CODE  0xFD

//The amount of times that the IR sensor will be read and has to be 
//consistent before returning a sensor detected value. Used in
//On_Pod_Change(). This is used to prevent false detections.
#define POLL_AMOUNT         25

//LED Ring max and min values for various animations
//Added fade rate too
#define RING_MAX            300
#define RING_MIN            0
#define RING_FADE_RATE      80

//Time_Check(*tmark,delay) Delays 
//(Delay in Seconds = value * 0.008s per interrupt)
#define TIME_DELAY_1S       125
#define TIME_DELAY_2S       250
#define TIME_DELAY_3S       375
#define TIME_DELAY_4S       500
#define TIME_DELAY_5S       625
#define TIME_DELAY_6S       750
#define TIME_DELAY_7S       875
#define TIME_DELAY_8S       1000
#define TIME_DELAY_9S       1125
#define TIME_DELAY_10S      1250
#define TIME_DELAY_15S      1875
#define TIME_DELAY_20S      2500
#define TIME_DELAY_30S      3750
#define TIME_DELAY_40S      5000
#define TIME_DELAY_50S      6250
#define TIME_DELAY_1M       7500
#define TIME_DELAY_2M       15000
#define TIME_DELAY_3M       22500
#define TIME_DELAY_4M       30000
#define TIME_DELAY_5M       37500
#define TIME_DELAY_1H       450000L                           

/*************************************************
*                   Macros                       *
*************************************************/

/*************************************************
*              Function Prototypes               *
*************************************************/ 
void Fade_State(void);
void Set_All_Pods(RGB color);
void Startup_Animation(void);
void Reset_All_Variables(void);  
void Display_Error(UINT8 error);
void Animate_On_Detection(void);
void Reset_Sequences(UINT8 mode);  
void Dual_Wave_On_Detection(void);
void LED_no(UINT8 LED, UINT16 data);
void Fade_Out_Table(UINT16 fade_rate);
void Ball_Washers_Detect(UINT8 detection);
void Ball_Washer_Jam_Error(UINT8 error_code);
void Pod_Set_Color(UINT8 pod, RGB pod_color);  
void Fade_All_Pods(RGB NEW, UINT16 fade_rate);       
void Set_All_Rings(UINT16 duty_cycle, UINT8 mode);
void Fade_To(UINT8 pod, RGB OLD, RGB NEW, UINT16 delay);
void Fade_Ring(UINT8 LED, float old, float new, UINT16 delay);
void Pod_Detect(UINT32 detection, RGB off_color, RGB on_color);
void RGB_Pod(UINT8 pod, UINT16 red, UINT16 green, UINT16 blue);            
void Ball_Washer(UINT8 bw, UINT16 fan_speed, UINT16 pump_speed);
void Fade_All_Rings(UINT16 duty_cycle, UINT16 fade_rate, UINT8 mode);

UINT8 Fade_Cups(void);
UINT8 Ring_Chase(void);
UINT8 Cycle_Rings(void); 
INT8 On_Pod_Change(void);
UINT8 Pyramid_Chase(void);
UINT8 Our_Test_Animation(void);
UINT8 Animation_On_Change(void);
UINT8 Cycle_Pod_Animations(void);
UINT8 Cycle_Grid_Animations(void);
UINT8 Cycle_Ring_Animations(void);
UINT8 Cycle_Combined_Animations(void);
UINT8 Ripple_Out(UINT16 fade_rate, UINT16 delay);
UINT8 Color_Throb(RGB outside_color, RGB inside_color);


#endif
