/*******************************************************************************
* Title: RF_Controls.c                                                                      
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                                       
*                                                                              
* Description:                                                                 
* This file contains the functions and commands that are necessary in order to                                                                             
* be able to communicate between the beer pong table and a PC.                                                                             
*******************************************************************************/

#ifndef RF_CONTROLS_C
#define RF_CONTROLS_C

#include "Main_Includes.h"
#include "RF_Controls.h"
#include "TLC5940_Setup.h"
#include "Miscellaneous.h"
#include "LED_Graphics.h"
#include "HT1632C_Setup.h"
#include "nRF24L01_Setup.h"

/*************************************************
*               Global Variables                 *
*************************************************/
extern volatile UINT8 scroll_delay;
extern volatile UINT8 RGB_data[120];
extern volatile UINT8 grid_brightness;

extern volatile T16_FLAG FLAG1;
extern volatile UINT16 LEDring[16];
extern volatile UINT16 LED_data[32]; 

/*******************************************************************************
* Function: Rx_Check_CMD(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will read the received data from the nRF24L01+ and parse it to
* determine what command is to be carried out.                                                                             
*******************************************************************************/
void Rx_Check_CMD(void)
{ 
  UINT16 CMD;
	UINT8 data[32];

	//Reset the scroll finished flag in case it has been set so that we can
	//scroll text if we want too
	SCROLL_FINISHED = 0;

	//Clear the RX_DR flag
	nRF24L01_Clear_IRQ(RF_CONFIG_MASK_RX_DR);
	
  //Read in the data from the nRF24L01+
  nRF24L01_Rx(data,32);  
  
  //Pack the first 2 data bytes which are the COMMAND bytes
  CMD = ((UINT16)data[1] << 8) | data[0];
  
  //Determine which command is to be carried out and point to the 30 bytes
  //of raw data for the specified function (Do not use the CMD bytes (B0 & B1))
  switch (CMD)
  {
	  case CMD_REFRESH_SCREEN:    Rx_Refresh_Grid(&data[2]);     break;
	  case CMD_UPDATE_PODS:       Rx_Update_Pods(&data[2]);      break;
	  case CMD_UPDATE_RINGS:      Rx_Update_Rings(&data[2]);     break;
	  case CMD_SCROLLING_TEXT:    Rx_Scrolling_Text(&data[2]);   break;
	  case CMD_ACTIVATE_RF_MODE:  Rx_Activate_RF_Mode(&data[2]); break;
	  case CMD_DISABLE_RF_MODE:   Rx_Disable_RF_Mode(&data[2]);  break;
	} 
}

/*******************************************************************************
* Function: Rx_Activate_RF_Mode(UINT8 *data)                            
*                                                                             
* Variables:    
* *data -> No data is used here yet. Left it in place for future development.
*                                                                             
* Description:                                                                
* This function will set RF_ACTIVE so that the beer pong table knows that it is
* in RF mode.                                                   
*******************************************************************************/          
UINT8 Rx_Activate_RF_Mode(UINT8 *data)
{
  //Enter RF mode
  RF_ACTIVE = 1;
   
  //Operation successful, return a 0
  return 0x00;
}

/*******************************************************************************
* Function: Rx_Check_For_Data(void)                             
*                                                                             
* Variables:    
* N/A
*                                                                             
* Description:                                                                
* This function will read the STATUS bits of the nRF24L01+ to see if any data has 
* been received. I normally would have used the RF_IRQ interrupt pin to check for
* data, but it wasn't working properly for me. I was forced to actually read the
* STATUS register instead.                                               
*******************************************************************************/          
void Rx_Check_For_Data(void)
{	 
  UINT8 status;
   
	//Disable the TLC5940's and the XLAT interrupts so that the SPI line
  //Can be dedicated for the RF module	 
	_T1IE = 0;
	TLC_ENA = 1;	
	
	//Read in the STATUS bits
	nRF24L01_ReadReg(RF_STATUS,&status,1);
	
	//If the next packet has been received, check what CMD was sent
	if (status & RF_STATUS_RX_DR)
		Rx_Check_CMD();
	
  //Enable the TLC5940's and the XLAT interrupts so that the SPI line
  //can be used for the RGB pods, ball washer motors and LED rings again
	_T1IE = 1;
	TLC_ENA = 0;
}

/*******************************************************************************
* Function: Rx_Initialize_RF_Module(void)                             
*                                                                             
* Variables:    
* N/A
*                                                                             
* Description:                                                                
* This function is used to initialize the nRF24L01+ and set its TX and RX
* addresses. These values must match the settings of the RF Master Board to
* enable communication.                                                
*******************************************************************************/          
void Rx_Initialize_RF_Module(void)
{
  UINT8 pipe0[5] = {0xA0,0x00,0x00,0x00,0x00}; 
  UINT8 pipe1[5] = {0x01,0x12,0x12,0x12,0x3F}; 
  UINT8 rx_payloadi[6] = {32,32,32,32,32,32};
  
  //Disable the TLC5940's and the XLAT interrupts so that the SPI line
  //Can be dedicated for the RF module		
  _T1IE = 0;
	TLC_ENA = 1;
	
	//Initialize the nRF24L01 in RX mode, turn on enhanced shockburst
	//and enable Enhanced Shockburst, set transmission speed to 2Mbps   
  nRF24L01_Init(0x39,1,0x3F,0x3F,0x03,0x2F,0x02,0x07);
  
  //Set up the RX addresses
  nRF24L01_SetRxAddress(pipe0,pipe1,5,rx_payloadi);
  
  //Set up the TX addresses
  nRF24L01_SetTxAddress(pipe0,5); 
  
  //Enable the TLC5940's and the XLAT interrupts so that the SPI line
  //can be used for the RGB pods, ball washer motors and LED rings again		
	_T1IE = 1;
	TLC_ENA = 0;
}

/*******************************************************************************
* Function: Rx_Disable_RF_Mode(UINT8 *data)                                
*                                                                             
* Variables:  
* *data -> No data is used here yet. Left it in place for future development.
*                                                                             
* Description:                                                                
* This function will clear RF_ACTIVE so that the beer pong table knows that it is
* no longer in RF mode.                                        
*******************************************************************************/          
UINT8 Rx_Disable_RF_Mode(UINT8 *data)
{
  //Exit RF mode
  RF_ACTIVE = 0;
   
  //Operation successful, return a 0
  return 0x00;
}

/*******************************************************************************
* Function: Rx_Refresh_Grid(UINT8 *data)                                      
*                                                                             
* Variables:                                                                  
* *data -> The data that is to be written to the display           
*                                                                             
* Description:                                                                
* This function will parse the data that has been received for the BPT and update
* the LED grid with the new data. If either of the three packets take too long to
* receive, the function will timeout and return an error.                                                   
*******************************************************************************/          
UINT8 Rx_Refresh_Grid(UINT8 *data)
{
  UINT16 j;
  UINT8 i;
  UINT8 k = 0;
  UINT8 status;
  UINT8 tracker = 0;
  UINT8 rx_data[32];
  
  //Store the first 30 bytes to be written to the LED grid
  for (i = 0;i < 15;i++)
  {
    //Pack two consecutive bytes into one WORD in little endian format
    k = i * 2;
    LED_data[i] = ((UINT16)data[k+1] << 8) | data[k];
  }
  
  //Reset 'j' 
  j = 0;
  
  //Keep looping and checking for the next packet of data; If no data after 6000 loops,
  //return a time-out error  
  do
  {
    //Read STATUS
    nRF24L01_ReadReg(RF_STATUS,&status,1);
    
    //Check to see if the next packet has been received
    if (status & RF_STATUS_RX_DR)
    {
      //If the 2nd packet has been recevied, parse the data
      if (tracker == 0)
      {
        //Read in the second packet
        nRF24L01_Rx(&rx_data[0],32);
        
        //Store the next 32 bytes to be written to the LED grid
        for (i = 15;i < 31;i++)
        {
          //Pack two consecutive bytes into one WORD in little endian format
          k = ((i - 15) * 2);
          LED_data[i] = ((UINT16)rx_data[k+1] << 8) | rx_data[k];
        }
        
        //Let the program know that it should poll for the third packet
        tracker++;
      }
          
      //If the 3rd packet has been recevied, parse the data
      else if (tracker == 1)
      {
        //Read in the third packet
        nRF24L01_Rx(&rx_data[0],32);
        
        //Store the last 2 bytes to be written to the LED grid;Little Endian
	      LED_data[31] = ((UINT16)rx_data[1] << 8) | rx_data[0];
	      
	      //Reset tracker  
        tracker = 0;
        
        //Update the LED grid
	  	  HT1632_UPDATE = 1;
	      
	      //Return operation successful
	      return 0x00;  
      }    
    }  
    
    //Increment the timeout counter     
    j++;
  }
  while (j < 6000);
   
  //Operation timed-out, return error 
  return 0xFF;
}

/*******************************************************************************
* Function: Rx_Scrolling_Text(UINT8 *data)                                
*                                                                             
* Variables:   
* *data -> The data that is to be included in the scrolling text 
*                                                                             
* Description:                                                                
* This function will parse the received data from the RF Master Board and display
* the text across the LED grid. There can be a total of 62 characters (including
* the null terminator) scrolled across the grid with this function. If either of 
* the packets take too long, the function will time out and return an error.                                                     
*******************************************************************************/          
UINT8 Rx_Scrolling_Text(UINT8 *data)
{
  UINT8 status;
  UINT8 rx_data[62];
  UINT16 j;
  
  //Parse the data in the first packet
  for (j = 0;j < 30;j++)
    rx_data[j] = data[j];
  
  //Reset 'j' 
  j = 0;
  
  //Keep looping and checking for the next packet of data; If no data after 6000 loops,
  //return a time-out error  
  do
  {
    //Read the STATUS bits
    nRF24L01_ReadReg(RF_STATUS,&status,1);
    
    //Checkt to see if the next packet has been received
    if (status & RF_STATUS_RX_DR)
    {
      //Retrieve the received data
      nRF24L01_Rx(&rx_data[30],32);
      
      //Make sure that the last character in the string is terminated in case
      //the user is trying to send too much text. Don't exceed the array size.
      rx_data[61] = '\0';
      
      //Begin scrolling the text across the LED grid
      Set_Scrolling_Text(rx_data);
      
    //Return operation successful
      return 0x00;  
    }
    //Increment the timeout counter     
    j++;
  }
  while (j < 6000);
   
  //Operation timed-out, return error 
  return 0xFF;
} 

/*******************************************************************************
* Function: Rx_Update_Pods(UINT8 *data)                                
*                                                                             
* Variables:                        
* *data -> The data that is to be written to the RGB pods   
*                                                                             
* Description:                                                                
* This function will parse the received data from the Master RF Board, convert
* the 8-bit RGB values to 12-bit RGB values and update the RGB pods. This
* function will parse the data from two 32 byte packets. If either of the packets
* take too long the function will time out and return an error.                                                  
*******************************************************************************/          
UINT8 Rx_Update_Pods(UINT8 *data)
{
  UINT8 i;
  UINT8 k;
  UINT8 status;
  UINT8 rx_data[62];
  
  UINT16 j;
  UINT16 red;
  UINT16 green;
  UINT16 blue;
  
  //Parse the data in the first packet
  for (i = 0;i < 30;i++)
    rx_data[i] = data[i];
  
  //Reset 'j' 
  j = 0;
  
  //Keep looping and checking for the next packet of data; If no data after 6000 loops,
  //return a time-out error  
  do
  {
    //Read STATUS bits
    nRF24L01_ReadReg(RF_STATUS,&status,1);
    
    //Check to see if the next packet has been received
    if (status & RF_STATUS_RX_DR)
    {
      //Read in the second packet
      nRF24L01_Rx(&rx_data[30],32);
      
      //Parse the data and update the RGB pods  
      for (i = 0;i < 20;i++)
      {
        //Index the array location for the specified pod
        k = (i*3);
        
        //Convert 8-bit RGB values to 12-bit RGB values
        red = rx_data[k] << 4;
        green = rx_data[k+1] << 4;
       	blue = rx_data[k+2] << 4;
        
        //Update the RGB pod
        RGB_Pod(i+1,red,green,blue);          
      } 
      
    //Return operation successful
      return 0x00;  
    }
    //Increment the timeout counter     
    j++;
  }
  while (j < 6000);
   
  //Operation timed-out, return error 
  return 0xFF;
}

/*******************************************************************************
* Function: Rx_Update_Rings(UINT8 *data)                                
*                                                                             
* Variables:  
* *data -> The data that is to be written to the LED rings 
*                                                                             
* Description:                                                                
* This function will parse data from the Master RF Board and set the LED rings
* PWM values according to the data that was received. The scroll speed variable
* and the grid brightness variable are also included in this packet. Byte #24 
* pertains to the scroll_delay variable and Byte #25 is for the grid_brightness
* variable.                                                   
*******************************************************************************/          
UINT8 Rx_Update_Rings(UINT8 *data)
{
  UINT8 i;
  UINT8 k;
  UINT16 value;
  
  //Parse the LED ring data
  for (i = 0;i < 12;i++)
  {
   //Pack two consecutive bytes into one WORD in little endian format
    k = i * 2;
    value = ((UINT16)data[k+1] << 8) | data[k];
    
    //Update the LED Ring
    LED_no(i+1,value);
  }
  
  //Save the scroll speed 
  scroll_delay = 15-data[24];
  
  //Save the grid brightness
  grid_brightness = data[25]; 
  
  //Update the grid brightness on the table 
  HT1632_Set_PWM(grid_brightness); 
   
  //Operation successful, return a 0
  return 0x00;
}  

#endif
