/*******************************************************************************
* Title: TLC5940_Setup.c                                                       
* Version: 1.0                                                                 
* Author: Jeff Nybo                                                            
* Date: April 23, 2014                                                                          
*                                                                              
* Description:                                           
* This file contains the functions and commands that are used to address and                                                                             
* control a specified amount of TLC5940's.                                                             
*******************************************************************************/
   
#ifndef TLC5940_SETUP_C
#define TLC5940_SETUP_C

#include "Main_Includes.h"
#include "PWM_Setup.h"
#include "Timers_Setup.h"
#include "TLC5940_Setup.h"
#include "SPI_Setup.h"

/*************************************************
*               Global Variables                 *
*************************************************/
extern volatile UINT8 RGB_data[120];
extern volatile UINT8 RGB_data2[120];
extern volatile T16_FLAG FLAG1;
  
/*******************************************************************************
* Function: Clear_Outputs(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will clear all of the outputs to the value of 0.                                                                             
*******************************************************************************/
void Clear_Outputs(void)
{
  UINT16 i;
  
  //Clear all outputs
  for (i = 0;i < (TLC_CHIPS * 24);i++)
    RGB_data[i] = 0;    
  
  //Update TLC5940's 
  TLC5940_Update(); 
}
  
/*******************************************************************************
* Function: Dot_Correction(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will set the dot correction for the TLC5940's. I just set default
* values.                                                                             
*******************************************************************************/
void Dot_Correction(void)
{
  UINT16 i;
  UINT16 j;
  
  //Prepare the chips to receive Dot Correction data
  TLC_DCPRG = 1;
  TLC_VPRG = 1;
  TLC_BLANK = 1;
  TLC_MOSI = 0;
  TLC_XLAT = 0; 
  
  //Write the default Dot Correction values into the chips  
  for (i = 0;i < (TLC_CHIPS *16);i++)
  {
    for (j = 0;j < 6;j++)
    {
      TLC_MOSI = 1;
      PULSE(TLC_SCK);
    }
  }
  
  //Latch in the new data  
  PULSE(TLC_XLAT);      
}
  
/*******************************************************************************
* Function: Fill_Outputs(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function will turn on all of the TLC5940's outputs at maximum power (4096).                                                                             
*******************************************************************************/
void Fill_Outputs(void)
{
  UINT16 i;
  
  //Set all outputs to maximum power
  for (i = 0;i < (TLC_CHIPS * 24);i++)
    RGB_data[i] = 255;    
 
  //Update TLC5940's
  TLC5940_Update(); 
}
  
/*******************************************************************************
* Function: Set_Initial_Grayscale(void)                                                                   
*                                                                             
* Variables:                                                                  
* N/A                                                                         
*                                                                             
* Description:                                                                
* This function only runs once. It will do the initial Grayscale setting which
* requires one extra clock pulse. This cannot be achieved with SPI and is manually
* implemented.                                                                            
*******************************************************************************/
void Set_Initial_Grayscale(void)
{
  UINT16 i;
 
  //Prepare to write first grayscale
  PULSE(TLC_XLAT);
  TLC_VPRG = 0;
  
  //Write all 192 grayscale bits
  for (i = 0;i < (TLC_CHIPS * 192);i++)
  {
    TLC_MOSI = 1;
    PULSE(TLC_SCK);
  }    
  
  //Latch data and provide extra clock pulse that is needed for first run
  PULSE(TLC_XLAT); 
  PULSE(TLC_SCK); 
  
  //Enable outputs on TLC5940's and begin next cycle
  TLC_BLANK = 0;    
}

/*******************************************************************************
* Function: TLC5940_Init(void)                                                 
*                                                                             
* Variables:                                                                  
* N/A                                                                         
*                                                                             
* Description:                                                                
* This function will prepare the TLC5940 to start accepting data.             
*******************************************************************************/
void TLC5940_Init(void)
{
  //Initialize necessary pins on the TLC5940
  TLC_GSCLK = 0;
  TLC_MOSI = 0;
  TLC_SCK = 0;
  TLC_DCPRG = 0;
  TLC_VPRG = 1;
  TLC_XLAT = 0;
  TLC_BLANK = 1;
  TLC_ENA = 0;
  
  //Set flags to their default values
  WAIT_XLAT = 0;
  TLC_UPDATE = 0;
}             
  
/*******************************************************************************
* Function: TLC5940_Update(void)                                                                   
*                                                                             
* Variables:                                                                  
* N/A                                                                         
*                                                                             
* Description:                                                                
* This function will copy the current RGB_data into a second buffer. This method 
* is used to prevent an interrupt happening in the midst of a function that is 
* only halfway through updating the first RGB_data buffer.                                                                            
*******************************************************************************/
void TLC5940_Update(void)
{
  UINT16 i;
  
  //Buffer data
  for (i = 0;i < (TLC_CHIPS * 24);i++)
     RGB_data2[i] = RGB_data[i];   
  
  //Prepare to update TLC5940's
  TLC_UPDATE = 1;   
}

/*******************************************************************************
* Function: XLAT_Interrupt(void)                                                                    
*                                                                              
* Variables:                                                                   
* N/A                                                                          
*                                                                              
* Description:                                                                 
* This function is used in part with a timer interrupt. There are 4096 steps
* per grayscale cycle for the TLC5940's. We want to keep the outputs updated at
* a rate of 60Hz, therefore (60Hz * 4096 steps) = 245760Hz which is close enough
* for us to use 250KHz. We must set a PWM on GSCLK operating at 250KHz and have a
* timer that interrupts every 60Hz (16384us approximately) and calls this function.                                                                              
*******************************************************************************/
void XLAT_Interrupt(void)
{
  UINT16 i;
  
  //Disable outputs, prepare for new cycle
  TLC_BLANK = 1;
  
  //Check if there is new data to be latched in
  if (WAIT_XLAT)
    PULSE(TLC_XLAT); 
  
  //Clear the XLAT flag and re-enable the outputs
  WAIT_XLAT = 0;
  TLC_BLANK = 0;
  
  //Check if new data needs to be shifted in  
  if (TLC_UPDATE)
  {    
    for (i = 0;i < (TLC_CHIPS * 24);i++)
        SPI2_Send(RGB_data2[i]);  
  }  
  
  //Clear update flag and set XLAT flag      
  TLC_UPDATE = 0;
  WAIT_XLAT = 1;
} 

#endif
